@@ -983,11 +983,11 @@ static int lxcContainerMountBasicFS(bool userns_enabled,
          */
         bindOverReadonly = !!(mnt_mflags & MS_RDONLY);
 
-        VIR_DEBUG("Mount %s on %s type=%s flags=%x",
+        VIR_DEBUG("Mount %s on %s type=%s flags=0x%x",
                   mnt_src, mnt->dst, mnt->type, mnt_mflags & ~MS_RDONLY);
         if (mount(mnt_src, mnt->dst, mnt->type, mnt_mflags & ~MS_RDONLY, NULL) < 0) {
             virReportSystemError(errno,
-                                 _("Failed to mount %s on %s type %s flags=%x"),
+                                 _("Failed to mount %s on %s type %s flags=0x%x"),
                                  mnt_src, mnt->dst, NULLSTR(mnt->type),
                                  mnt_mflags & ~MS_RDONLY);
             goto cleanup;
@@ -997,7 +997,7 @@ static int lxcContainerMountBasicFS(bool userns_enabled,
             mount(mnt_src, mnt->dst, NULL,
                   MS_BIND|MS_REMOUNT|mnt_mflags|MS_RDONLY, NULL) < 0) {
             virReportSystemError(errno,
-                                 _("Failed to re-mount %s on %s flags=%x"),
+                                 _("Failed to re-mount %s on %s flags=0x%x"),
                                  mnt_src, mnt->dst,
                                  MS_BIND|MS_REMOUNT|MS_RDONLY);
             goto cleanup;
@@ -2035,7 +2035,7 @@ static int lxcContainerDropCapabilities(virDomainDefPtr def,
             break;
 
         case VIR_DOMAIN_CAPABILITIES_POLICY_DEFAULT:
-            switch ((virDomainCapsFeature) i) {
+            switch (i) {
             case VIR_DOMAIN_CAPS_FEATURE_SYS_BOOT: /* No use of reboot */
                 toDrop = !keepReboot && (state != VIR_TRISTATE_SWITCH_ON);
                 break;
@@ -2066,10 +2066,10 @@ static int lxcContainerDropCapabilities(virDomainDefPtr def,
             }
             break;
 
+        case VIR_DOMAIN_CAPABILITIES_POLICY_LAST:
         default:
-            virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
-                           _("Unsupported capabilities policy: %s"),
-                           virDomainCapabilitiesPolicyTypeToString(policy));
+            virReportEnumRangeError(virDomainCapabilitiesPolicy, policy);
+            return -1;
         }
     }
 
@@ -2159,6 +2159,37 @@ static int lxcContainerSetUserGroup(virCommandPtr cmd,
     return 0;
 }
 
+static const char hostname_validchars[] =
+    "abcdefghijklmnopqrstuvwxyz"
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    "0123456789-";
+
+static int lxcContainerSetHostname(virDomainDefPtr def)
+{
+    int ret = -1;
+    char *name = NULL;
+    char *hostname = NULL;
+
+    /* Filter the VM name to get a valid hostname */
+    if (VIR_STRDUP(name, def->name) < 0)
+        goto cleanup;
+
+    /* RFC 1123 allows 0-9 digits as a first character in hostname */
+    virStringFilterChars(name, hostname_validchars);
+    hostname = name;
+    if (strlen(name) > 0 && name[0] == '-')
+        hostname = name + 1;
+
+    if (sethostname(hostname, strlen(hostname)) < 0) {
+        virReportSystemError(errno, "%s", _("Failed to set hostname"));
+        goto cleanup;
+    }
+    ret = 0;
+
+ cleanup:
+    VIR_FREE(name);
+    return ret;
+}
 
 /**
  * lxcContainerChild:
@@ -2167,7 +2198,7 @@ static int lxcContainerSetUserGroup(virCommandPtr cmd,
  * This function is run in the process clone()'d in lxcStartContainer.
  * Perform a number of container setup tasks:
  *     Setup container file system
- *     mount container /proca
+ *     mount container /proc
  * Then exec's the container init
  *
  * Returns 0 on success or -1 in case of error
@@ -2182,6 +2213,8 @@ static int lxcContainerChild(void *data)
     virDomainFSDefPtr root;
     virCommandPtr cmd = NULL;
     int hasReboot;
+    gid_t *groups = NULL;
+    int ngroups;
 
     if (NULL == vmDef) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
@@ -2252,8 +2285,8 @@ static int lxcContainerChild(void *data)
 
     if (!virFileExists(vmDef->os.init)) {
         virReportSystemError(errno,
-                    _("cannot find init path '%s' relative to container root"),
-                    vmDef->os.init);
+                             _("cannot find init path '%s' relative to container root"),
+                             vmDef->os.init);
         goto cleanup;
     }
 
@@ -2267,13 +2300,17 @@ static int lxcContainerChild(void *data)
         goto cleanup;
     }
 
+    if (lxcContainerSetHostname(vmDef) < 0)
+        goto cleanup;
+
+
     /* drop a set of root capabilities */
     if (lxcContainerDropCapabilities(vmDef, !!hasReboot) < 0)
         goto cleanup;
 
     if (lxcContainerSendContinue(argv->handshakefd) < 0) {
         virReportSystemError(errno, "%s",
-                            _("Failed to send continue signal to controller"));
+                             _("Failed to send continue signal to controller"));
         goto cleanup;
     }
 
@@ -2297,6 +2334,13 @@ static int lxcContainerChild(void *data)
         goto cleanup;
     }
 
+    /* TODO is it safe to call it here or should this call be moved in
+     * front of the clone() as otherwise there might be a risk for a
+     * deadlock */
+    if ((ngroups = virGetGroupList(virCommandGetUID(cmd), virCommandGetGID(cmd),
+                                   &groups)) < 0)
+        goto cleanup;
+
     ret = 0;
  cleanup:
     VIR_FREE(ttyPath);
@@ -2307,7 +2351,7 @@ static int lxcContainerChild(void *data)
     if (ret == 0) {
         VIR_DEBUG("Executing init binary");
         /* this function will only return if an error occurred */
-        ret = virCommandExec(cmd);
+        ret = virCommandExec(cmd, groups, ngroups);
     }
 
     if (ret != 0) {
@@ -2317,6 +2361,7 @@ static int lxcContainerChild(void *data)
                 virGetLastErrorMessage());
     }
 
+    VIR_FREE(groups);
     virCommandFree(cmd);
     return ret;
 }
