@@ -30,7 +30,6 @@
 #include "node_device_conf.h"
 #include "node_device_event.h"
 #include "node_device_driver.h"
-#include "node_device_linux_sysfs.h"
 #include "node_device_udev.h"
 #include "virerror.h"
 #include "driver.h"
@@ -53,12 +52,78 @@ VIR_LOG_INIT("node_device.node_device_udev");
 # define TYPE_RAID 12
 #endif
 
-struct _udevPrivate {
+typedef struct _udevEventData udevEventData;
+typedef udevEventData *udevEventDataPtr;
+
+struct _udevEventData {
+    virObjectLockable parent;
+
     struct udev_monitor *udev_monitor;
     int watch;
-    bool privileged;
+
+    /* Thread data */
+    virThread th;
+    virCond threadCond;
+    bool threadQuit;
+    bool dataReady;
 };
 
+static virClassPtr udevEventDataClass;
+
+static void
+udevEventDataDispose(void *obj)
+{
+    struct udev *udev = NULL;
+    udevEventDataPtr priv = obj;
+
+    if (priv->watch != -1)
+        virEventRemoveHandle(priv->watch);
+
+    if (!priv->udev_monitor)
+        return;
+
+    udev = udev_monitor_get_udev(priv->udev_monitor);
+    udev_monitor_unref(priv->udev_monitor);
+    udev_unref(udev);
+
+    virCondDestroy(&priv->threadCond);
+}
+
+
+static int
+udevEventDataOnceInit(void)
+{
+    if (!(udevEventDataClass = virClassNew(virClassForObjectLockable(),
+                                           "udevEventData",
+                                           sizeof(udevEventData),
+                                           udevEventDataDispose)))
+        return -1;
+
+    return 0;
+}
+
+VIR_ONCE_GLOBAL_INIT(udevEventData)
+
+static udevEventDataPtr
+udevEventDataNew(void)
+{
+    udevEventDataPtr ret = NULL;
+
+    if (udevEventDataInitialize() < 0)
+        return NULL;
+
+    if (!(ret = virObjectLockableNew(udevEventDataClass)))
+        return NULL;
+
+    if (virCondInit(&ret->threadCond) < 0) {
+        virObjectUnref(ret);
+        return NULL;
+    }
+
+    ret->watch = -1;
+    return ret;
+}
+
 
 static bool
 udevHasDeviceProperty(struct udev_device *dev,
@@ -327,119 +392,6 @@ udevTranslatePCIIds(unsigned int vendor,
 }
 
 
-static int
-udevFillMdevType(struct udev_device *device,
-                 const char *dir,
-                 virNodeDevCapMdevTypePtr type)
-{
-    int ret = -1;
-    char *attrpath = NULL;
-
-#define MDEV_GET_SYSFS_ATTR(attr_name, cb, ...)                             \
-    do {                                                                    \
-        if (virAsprintf(&attrpath, "%s/%s", dir, #attr_name) < 0)           \
-            goto cleanup;                                                   \
-                                                                            \
-        if (cb(device, attrpath, __VA_ARGS__) < 0)                          \
-            goto cleanup;                                                   \
-                                                                            \
-        VIR_FREE(attrpath);                                                 \
-    } while (0)                                                             \
-
-    if (VIR_STRDUP(type->id, last_component(dir)) < 0)
-        goto cleanup;
-
-    /* query udev for the attributes under subdirectories using the relative
-     * path stored in @dir, i.e. 'mdev_supported_types/<type_id>'
-     */
-    MDEV_GET_SYSFS_ATTR(name, udevGetStringSysfsAttr, &type->name);
-    MDEV_GET_SYSFS_ATTR(device_api, udevGetStringSysfsAttr, &type->device_api);
-    MDEV_GET_SYSFS_ATTR(available_instances, udevGetUintSysfsAttr,
-                        &type->available_instances, 10);
-
-#undef MDEV_GET_SYSFS_ATTR
-
-    ret = 0;
- cleanup:
-    VIR_FREE(attrpath);
-    return ret;
-}
-
-
-static int
-udevPCIGetMdevTypesCap(struct udev_device *device,
-                       virNodeDevCapPCIDevPtr pcidata)
-{
-    int ret = -1;
-    int dirret = -1;
-    DIR *dir = NULL;
-    struct dirent *entry;
-    char *path = NULL;
-    char *tmppath = NULL;
-    virNodeDevCapMdevTypePtr type = NULL;
-    virNodeDevCapMdevTypePtr *types = NULL;
-    size_t ntypes = 0;
-    size_t i;
-
-    if (virAsprintf(&path, "%s/mdev_supported_types",
-                    udev_device_get_syspath(device)) < 0)
-        return -1;
-
-    if ((dirret = virDirOpenIfExists(&dir, path)) < 0)
-        goto cleanup;
-
-    if (dirret == 0) {
-        ret = 0;
-        goto cleanup;
-    }
-
-    if (VIR_ALLOC(types) < 0)
-        goto cleanup;
-
-    /* UDEV doesn't report attributes under subdirectories by default but is
-     * able to query them if the path to the attribute is relative to the
-     * device's base path, e.g. /sys/devices/../0000:00:01.0/ is the device's
-     * base path as udev reports it, but we're interested in attributes under
-     * /sys/devices/../0000:00:01.0/mdev_supported_types/<type>/. So, we need to
-     * scan the subdirectories ourselves.
-     */
-    while ((dirret = virDirRead(dir, &entry, path)) > 0) {
-        if (VIR_ALLOC(type) < 0)
-            goto cleanup;
-
-        /* construct the relative mdev type path bit for udev */
-        if (virAsprintf(&tmppath, "mdev_supported_types/%s", entry->d_name) < 0)
-            goto cleanup;
-
-        if (udevFillMdevType(device, tmppath, type) < 0)
-            goto cleanup;
-
-        if (VIR_APPEND_ELEMENT(types, ntypes, type) < 0)
-            goto cleanup;
-
-        VIR_FREE(tmppath);
-    }
-
-    if (dirret < 0)
-        goto cleanup;
-
-    VIR_STEAL_PTR(pcidata->mdev_types, types);
-    pcidata->nmdev_types = ntypes;
-    pcidata->flags |= VIR_NODE_DEV_CAP_FLAG_PCI_MDEV;
-    ntypes = 0;
-    ret = 0;
- cleanup:
-    virNodeDevCapMdevTypeFree(type);
-    for (i = 0; i < ntypes; i++)
-        virNodeDevCapMdevTypeFree(types[i]);
-    VIR_FREE(types);
-    VIR_FREE(path);
-    VIR_FREE(tmppath);
-    VIR_DIR_CLOSE(dir);
-    return ret;
-}
-
-
 static int
 udevProcessPCI(struct udev_device *device,
                virNodeDeviceDefPtr def)
@@ -447,9 +399,13 @@ udevProcessPCI(struct udev_device *device,
     virNodeDevCapPCIDevPtr pci_dev = &def->caps->data.pci_dev;
     virPCIEDeviceInfoPtr pci_express = NULL;
     virPCIDevicePtr pciDev = NULL;
-    udevPrivate *priv = driver->privateData;
     int ret = -1;
     char *p;
+    bool privileged;
+
+    nodeDeviceLock();
+    privileged = driver->privileged;
+    nodeDeviceUnlock();
 
     if (udevGetUintProperty(device, "PCI_CLASS", &pci_dev->class, 16) < 0)
         goto cleanup;
@@ -488,7 +444,7 @@ udevProcessPCI(struct udev_device *device,
                             &pci_dev->numa_node, 10) < 0)
         goto cleanup;
 
-    if (nodeDeviceSysfsGetPCIRelatedDevCaps(def->sysfs_path, pci_dev) < 0)
+    if (virNodeDeviceGetPCIDynamicCaps(def->sysfs_path, pci_dev) < 0)
         goto cleanup;
 
     if (!(pciDev = virPCIDeviceNew(pci_dev->domain,
@@ -498,7 +454,7 @@ udevProcessPCI(struct udev_device *device,
         goto cleanup;
 
     /* We need to be root to read PCI device configs */
-    if (priv->privileged) {
+    if (privileged) {
         if (virPCIGetHeaderType(pciDev, &pci_dev->hdrType) < 0)
             goto cleanup;
 
@@ -527,12 +483,6 @@ udevProcessPCI(struct udev_device *device,
         }
     }
 
-    /* check whether the device is mediated devices framework capable, if so,
-     * process it
-     */
-    if (udevPCIGetMdevTypesCap(device, pci_dev) < 0)
-        goto cleanup;
-
     ret = 0;
 
  cleanup:
@@ -711,7 +661,7 @@ udevProcessSCSIHost(struct udev_device *device ATTRIBUTE_UNUSED,
         return -1;
     }
 
-    nodeDeviceSysfsGetSCSIHostCaps(&def->caps->data.scsi_host);
+    virNodeDeviceGetSCSIHostCaps(&def->caps->data.scsi_host);
 
     if (udevGenerateDeviceName(device, def, NULL) != 0)
         return -1;
@@ -732,7 +682,7 @@ udevProcessSCSITarget(struct udev_device *device,
     if (VIR_STRDUP(scsi_target->name, sysname) < 0)
         return -1;
 
-    nodeDeviceSysfsGetSCSITargetCaps(def->sysfs_path, &def->caps->data.scsi_target);
+    virNodeDeviceGetSCSITargetCaps(def->sysfs_path, &def->caps->data.scsi_target);
 
     if (udevGenerateDeviceName(device, def, NULL) != 0)
         return -1;
@@ -1118,11 +1068,27 @@ udevProcessMediatedDevice(struct udev_device *dev,
     char *canonicalpath = NULL;
     virNodeDevCapMdevPtr data = &def->caps->data.mdev;
 
-    if (virAsprintf(&linkpath, "%s/mdev_type", udev_device_get_syspath(dev)) < 0)
+    /* Because of a kernel uevent race, we might get the 'add' event prior to
+     * the sysfs tree being ready, so any attempt to access any sysfs attribute
+     * would result in ENOENT and us dropping the device, so let's work around
+     * it by waiting for the attributes to become available.
+     */
+
+    if (virAsprintf(&linkpath, "%s/mdev_type",
+                    udev_device_get_syspath(dev)) < 0)
         goto cleanup;
 
-    if (virFileResolveLink(linkpath, &canonicalpath) < 0)
+    if (virFileWaitForExists(linkpath, 1, 100) < 0) {
+        virReportSystemError(errno,
+                             _("failed to wait for file '%s' to appear"),
+                             linkpath);
         goto cleanup;
+    }
+
+    if (virFileResolveLink(linkpath, &canonicalpath) < 0) {
+        virReportSystemError(errno, _("failed to resolve '%s'"), linkpath);
+        goto cleanup;
+    }
 
     if (VIR_STRDUP(data->type, last_component(canonicalpath)) < 0)
         goto cleanup;
@@ -1557,88 +1523,155 @@ udevPCITranslateDeinit(void)
 static int
 nodeStateCleanup(void)
 {
-    udevPrivate *priv = NULL;
-    struct udev_monitor *udev_monitor = NULL;
-    struct udev *udev = NULL;
+    udevEventDataPtr priv = NULL;
 
     if (!driver)
         return -1;
 
-    nodeDeviceLock();
-
-    virObjectUnref(driver->nodeDeviceEventState);
-
     priv = driver->privateData;
-
     if (priv) {
-        if (priv->watch != -1)
-            virEventRemoveHandle(priv->watch);
-
-        udev_monitor = DRV_STATE_UDEV_MONITOR(driver);
-
-        if (udev_monitor != NULL) {
-            udev = udev_monitor_get_udev(udev_monitor);
-            udev_monitor_unref(udev_monitor);
-        }
+        virObjectLock(priv);
+        priv->threadQuit = true;
+        virCondSignal(&priv->threadCond);
+        virObjectUnlock(priv);
+        virThreadJoin(&priv->th);
     }
 
-    if (udev != NULL)
-        udev_unref(udev);
+    virObjectUnref(priv);
+    virObjectUnref(driver->nodeDeviceEventState);
 
     virNodeDeviceObjListFree(driver->devs);
-    nodeDeviceUnlock();
     virMutexDestroy(&driver->lock);
     VIR_FREE(driver);
-    VIR_FREE(priv);
 
     udevPCITranslateDeinit();
     return 0;
 }
 
 
+static int
+udevHandleOneDevice(struct udev_device *device)
+{
+    const char *action = udev_device_get_action(device);
+
+    VIR_DEBUG("udev action: '%s'", action);
+
+    if (STREQ(action, "add") || STREQ(action, "change"))
+        return udevAddOneDevice(device);
+
+    if (STREQ(action, "remove"))
+        return udevRemoveOneDevice(device);
+
+    return 0;
+}
+
+
+/* the caller must be holding the udevEventData object lock prior to calling
+ * this function
+ */
+static bool
+udevEventMonitorSanityCheck(udevEventDataPtr priv,
+                            int fd)
+{
+    int rc = -1;
+
+    rc = udev_monitor_get_fd(priv->udev_monitor);
+    if (fd != rc) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("File descriptor returned by udev %d does not "
+                         "match node device file descriptor %d"),
+                       fd, rc);
+
+        /* this is a non-recoverable error, let's remove the handle, so that we
+         * don't get in here again because of some spurious behaviour and report
+         * the same error multiple times
+         */
+        virEventRemoveHandle(priv->watch);
+        priv->watch = -1;
+
+        return false;
+    }
+
+    return true;
+}
+
+
+static void
+udevEventHandleThread(void *opaque ATTRIBUTE_UNUSED)
+{
+    udevEventDataPtr priv = driver->privateData;
+    struct udev_device *device = NULL;
+
+    /* continue rather than break from the loop on non-fatal errors */
+    while (1) {
+        virObjectLock(priv);
+        while (!priv->dataReady && !priv->threadQuit) {
+            if (virCondWait(&priv->threadCond, &priv->parent.lock)) {
+                virReportSystemError(errno, "%s",
+                                     _("handler failed to wait on condition"));
+                virObjectUnlock(priv);
+                return;
+            }
+        }
+
+        if (priv->threadQuit) {
+            virObjectUnlock(priv);
+            return;
+        }
+
+        errno = 0;
+        device = udev_monitor_receive_device(priv->udev_monitor);
+        virObjectUnlock(priv);
+
+        if (!device) {
+            if (errno == 0) {
+                virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                               _("failed to receive device from udev monitor"));
+                return;
+            }
+
+            /* POSIX allows both EAGAIN and EWOULDBLOCK to be used
+             * interchangeably when the read would block or timeout was fired
+             */
+            VIR_WARNINGS_NO_WLOGICALOP_EQUAL_EXPR
+            if (errno != EAGAIN && errno != EWOULDBLOCK) {
+            VIR_WARNINGS_RESET
+                virReportSystemError(errno, "%s",
+                                     _("failed to receive device from udev "
+                                       "monitor"));
+                return;
+            }
+
+            virObjectLock(priv);
+            priv->dataReady = false;
+            virObjectUnlock(priv);
+
+            continue;
+        }
+
+        udevHandleOneDevice(device);
+        udev_device_unref(device);
+    }
+}
+
+
 static void
 udevEventHandleCallback(int watch ATTRIBUTE_UNUSED,
                         int fd,
                         int events ATTRIBUTE_UNUSED,
                         void *data ATTRIBUTE_UNUSED)
 {
-    struct udev_device *device = NULL;
-    struct udev_monitor *udev_monitor = DRV_STATE_UDEV_MONITOR(driver);
-    const char *action = NULL;
-    int udev_fd = -1;
+    udevEventDataPtr priv = driver->privateData;
 
-    udev_fd = udev_monitor_get_fd(udev_monitor);
-    if (fd != udev_fd) {
-        virReportError(VIR_ERR_INTERNAL_ERROR,
-                       _("File descriptor returned by udev %d does not "
-                         "match node device file descriptor %d"),
-                       fd, udev_fd);
-        goto cleanup;
-    }
+    virObjectLock(priv);
 
-    device = udev_monitor_receive_device(udev_monitor);
-    if (device == NULL) {
-        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                       _("udev_monitor_receive_device returned NULL"));
-        goto cleanup;
-    }
+    if (!udevEventMonitorSanityCheck(priv, fd))
+        priv->threadQuit = true;
+    else
+        priv->dataReady = true;
 
-    action = udev_device_get_action(device);
-    VIR_DEBUG("udev action: '%s'", action);
-
-    if (STREQ(action, "add") || STREQ(action, "change")) {
-        udevAddOneDevice(device);
-        goto cleanup;
-    }
-
-    if (STREQ(action, "remove")) {
-        udevRemoveOneDevice(device);
-        goto cleanup;
-    }
-
- cleanup:
-    udev_device_unref(device);
-    return;
+    virCondSignal(&priv->threadCond);
+    virObjectUnlock(priv);
 }
 
 
@@ -1647,12 +1680,14 @@ udevEventHandleCallback(int watch ATTRIBUTE_UNUSED,
 static void
 udevGetDMIData(virNodeDevCapSystemPtr syscap)
 {
+    udevEventDataPtr priv = driver->privateData;
     struct udev *udev = NULL;
     struct udev_device *device = NULL;
     virNodeDevCapSystemHardwarePtr hardware = &syscap->hardware;
     virNodeDevCapSystemFirmwarePtr firmware = &syscap->firmware;
 
-    udev = udev_monitor_get_udev(DRV_STATE_UDEV_MONITOR(driver));
+    virObjectLock(priv);
+    udev = udev_monitor_get_udev(priv->udev_monitor);
 
     device = udev_device_new_from_syspath(udev, DMI_DEVPATH);
     if (device == NULL) {
@@ -1661,9 +1696,11 @@ udevGetDMIData(virNodeDevCapSystemPtr syscap)
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            _("Failed to get udev device for syspath '%s' or '%s'"),
                            DMI_DEVPATH, DMI_DEVPATH_FALLBACK);
+            virObjectUnlock(priv);
             return;
         }
     }
+    virObjectUnlock(priv);
 
     if (udevGetStringSysfsAttr(device, "product_name",
                                &syscap->product_name) < 0)
@@ -1734,6 +1771,25 @@ udevSetupSystemDev(void)
 }
 
 
+static void
+nodeStateInitializeEnumerate(void *opaque)
+{
+    struct udev *udev = opaque;
+    udevEventDataPtr priv = driver->privateData;
+
+    /* Populate with known devices */
+    if (udevEnumerateDevices(udev) != 0)
+        goto error;
+
+    return;
+
+ error:
+    virObjectLock(priv);
+    priv->threadQuit = true;
+    virObjectUnlock(priv);
+}
+
+
 static int
 udevPCITranslateInit(bool privileged ATTRIBUTE_UNUSED)
 {
@@ -1763,52 +1819,47 @@ nodeStateInitialize(bool privileged,
                     virStateInhibitCallback callback ATTRIBUTE_UNUSED,
                     void *opaque ATTRIBUTE_UNUSED)
 {
-    udevPrivate *priv = NULL;
+    udevEventDataPtr priv = NULL;
     struct udev *udev = NULL;
+    virThread enumThread;
 
-    if (VIR_ALLOC(priv) < 0)
+    if (VIR_ALLOC(driver) < 0)
         return -1;
 
-    priv->watch = -1;
-    priv->privileged = privileged;
-
-    if (VIR_ALLOC(driver) < 0) {
-        VIR_FREE(priv);
-        return -1;
-    }
-
     if (virMutexInit(&driver->lock) < 0) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                        _("Unable to initialize mutex"));
-        VIR_FREE(priv);
         VIR_FREE(driver);
         return -1;
     }
 
+    driver->privileged = privileged;
+
+    if (!(driver->devs = virNodeDeviceObjListNew()) ||
+        !(priv = udevEventDataNew()))
+        goto cleanup;
+
     driver->privateData = priv;
-    nodeDeviceLock();
-
-    if (!(driver->devs = virNodeDeviceObjListNew()))
-        goto unlock;
-
     driver->nodeDeviceEventState = virObjectEventStateNew();
 
     if (udevPCITranslateInit(privileged) < 0)
-        goto unlock;
+        goto cleanup;
 
     udev = udev_new();
     if (!udev) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                        _("failed to create udev context"));
-        goto unlock;
+        goto cleanup;
     }
 #if HAVE_UDEV_LOGGING
     /* cast to get rid of missing-format-attribute warning */
     udev_set_log_fn(udev, (udevLogFunctionPtr) udevLogFunction);
 #endif
 
+    virObjectLock(priv);
+
     priv->udev_monitor = udev_monitor_new_from_netlink(udev, "udev");
-    if (priv->udev_monitor == NULL) {
+    if (!priv->udev_monitor) {
         virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                        _("udev_monitor_new_from_netlink returned NULL"));
         goto unlock;
@@ -1825,6 +1876,12 @@ nodeStateInitialize(bool privileged,
                                              128 * 1024 * 1024);
 #endif
 
+    if (virThreadCreate(&priv->th, true, udevEventHandleThread, NULL) < 0) {
+        virReportSystemError(errno, "%s",
+                             _("failed to create udev handler thread"));
+        goto unlock;
+    }
+
     /* We register the monitor with the event callback so we are
      * notified by udev of device changes before we enumerate existing
      * devices because libvirt will simply recreate the device if we
@@ -1839,16 +1896,19 @@ nodeStateInitialize(bool privileged,
     if (priv->watch == -1)
         goto unlock;
 
+    virObjectUnlock(priv);
+
     /* Create a fictional 'computer' device to root the device tree. */
     if (udevSetupSystemDev() != 0)
-        goto unlock;
-
-    nodeDeviceUnlock();
-
-    /* Populate with known devices */
-    if (udevEnumerateDevices(udev) != 0)
         goto cleanup;
 
+    if (virThreadCreate(&enumThread, false, nodeStateInitializeEnumerate,
+                        udev) < 0) {
+        virReportSystemError(errno, "%s",
+                             _("failed to create udev enumerate thread"));
+        goto cleanup;
+    }
+
     return 0;
 
  cleanup:
@@ -1856,7 +1916,7 @@ nodeStateInitialize(bool privileged,
     return -1;
 
  unlock:
-    nodeDeviceUnlock();
+    virObjectUnlock(priv);
     goto cleanup;
 }
 
@@ -1885,6 +1945,23 @@ static virNodeDeviceDriver udevNodeDeviceDriver = {
     .nodeDeviceDestroy = nodeDeviceDestroy, /* 0.7.3 */
 };
 
+
+static virHypervisorDriver udevHypervisorDriver = {
+    .name = "nodedev",
+    .connectOpen = nodeConnectOpen, /* 4.1.0 */
+    .connectClose = nodeConnectClose, /* 4.1.0 */
+    .connectIsEncrypted = nodeConnectIsEncrypted, /* 4.1.0 */
+    .connectIsSecure = nodeConnectIsSecure, /* 4.1.0 */
+    .connectIsAlive = nodeConnectIsAlive, /* 4.1.0 */
+};
+
+
+static virConnectDriver udevConnectDriver = {
+    .hypervisorDriver = &udevHypervisorDriver,
+    .nodeDeviceDriver = &udevNodeDeviceDriver,
+};
+
+
 static virStateDriver udevStateDriver = {
     .name = "udev",
     .stateInitialize = nodeStateInitialize, /* 0.7.3 */
@@ -1898,6 +1975,8 @@ udevNodeRegister(void)
 {
     VIR_DEBUG("Registering udev node device backend");
 
+    if (virRegisterConnectDriver(&udevConnectDriver, false) < 0)
+        return -1;
     if (virSetSharedNodeDeviceDriver(&udevNodeDeviceDriver) < 0)
         return -1;
 
