@@ -65,6 +65,7 @@
 #include "virnetdev.h"
 #include "virfile.h"
 #include "viratomic.h"
+#include "virsocketaddr.h"
 #include "virthreadpool.h"
 #include "configmake.h"
 #include "virtime.h"
@@ -146,6 +147,7 @@ struct _virNWFilterSnoopReq {
     virNWFilterSnoopIPLeasePtr           start;
     virNWFilterSnoopIPLeasePtr           end;
     char                                *threadkey;
+    virErrorPtr                          threadError;
 
     virNWFilterSnoopThreadStatus         threadStatus;
     virCond                              threadStatusCond;
@@ -194,6 +196,7 @@ struct _virNWFilterSnoopEthHdr {
     uint16_t eh_type;
     uint8_t eh_data[];
 } ATTRIBUTE_PACKED;
+verify(sizeof(struct _virNWFilterSnoopEthHdr) == 14);
 
 typedef struct _virNWFilterSnoopDHCPHdr virNWFilterSnoopDHCPHdr;
 typedef virNWFilterSnoopDHCPHdr *virNWFilterSnoopDHCPHdrPtr;
@@ -215,6 +218,7 @@ struct _virNWFilterSnoopDHCPHdr {
     char      d_file[128];
     uint8_t   d_opts[];
 } ATTRIBUTE_PACKED;
+verify(sizeof(struct _virNWFilterSnoopDHCPHdr) == 236);
 
 /* DHCP options */
 
@@ -475,9 +479,6 @@ virNWFilterSnoopIPLeaseInstallRule(virNWFilterSnoopIPLeasePtr ipl,
     if (virNWFilterIPAddrMapAddIPAddr(req->ifname, ipaddr) < 0)
         goto exit_snooprequnlock;
 
-    /* ipaddr now belongs to the map */
-    ipaddr = NULL;
-
     if (!instantiate) {
         rc = 0;
         goto exit_snooprequnlock;
@@ -639,6 +640,7 @@ virNWFilterSnoopReqFree(virNWFilterSnoopReqPtr req)
 
     virMutexDestroy(&req->lock);
     virCondDestroy(&req->threadStatusCond);
+    virFreeError(req->threadError);
 
     VIR_FREE(req);
 }
@@ -1404,10 +1406,12 @@ virNWFilterDHCPSnoopThread(void *req0)
 
     /* let creator know how well we initialized */
     if (error || !threadkey || tmp < 0 || !worker ||
-        ifindex != req->ifindex)
+        ifindex != req->ifindex) {
+        virErrorPreserveLast(&req->threadError);
         req->threadStatus = THREAD_STATUS_FAIL;
-    else
+    } else {
         req->threadStatus = THREAD_STATUS_OK;
+    }
 
     virCondSignal(&req->threadStatusCond);
 
@@ -1713,9 +1717,16 @@ virNWFilterDHCPSnoopReq(virNWFilterTechDriverPtr techdriver,
     }
 
     /* sync with thread */
-    if (virCondWait(&req->threadStatusCond, &req->lock) < 0 ||
-        req->threadStatus != THREAD_STATUS_OK)
+    if (virCondWait(&req->threadStatusCond, &req->lock) < 0) {
+        virReportSystemError(errno, "%s",
+                             _("unable to wait on dhcp snoop thread"));
         goto exit_snoop_cancel;
+    }
+
+    if (req->threadStatus != THREAD_STATUS_OK) {
+        virErrorRestore(&req->threadError);
+        goto exit_snoop_cancel;
+    }
 
     virNWFilterSnoopReqUnlock(req);
 
