@@ -39,7 +39,7 @@
 #include "storage_backend_fs.h"
 #include "storage_util.h"
 #include "storage_conf.h"
-#include "virstoragefile.h"
+#include "virstoragefilebackend.h"
 #include "vircommand.h"
 #include "viralloc.h"
 #include "virxml.h"
@@ -145,8 +145,7 @@ virStorageBackendFileSystemNetFindNFSPoolSources(virNetfsDiscoverState *state)
 
 
 static char *
-virStorageBackendFileSystemNetFindPoolSources(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                              const char *srcSpec,
+virStorageBackendFileSystemNetFindPoolSources(const char *srcSpec,
                                               unsigned int flags)
 {
     virNetfsDiscoverState state = {
@@ -222,25 +221,27 @@ virStorageBackendFileSystemNetFindPoolSources(virConnectPtr conn ATTRIBUTE_UNUSE
 static int
 virStorageBackendFileSystemIsValid(virStoragePoolObjPtr pool)
 {
-    if (pool->def->type == VIR_STORAGE_POOL_NETFS) {
-        if (pool->def->source.nhost != 1) {
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
+
+    if (def->type == VIR_STORAGE_POOL_NETFS) {
+        if (def->source.nhost != 1) {
             virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
                            _("expected exactly 1 host for the storage pool"));
             return -1;
         }
-        if (pool->def->source.hosts[0].name == NULL) {
+        if (def->source.hosts[0].name == NULL) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            "%s", _("missing source host"));
             return -1;
         }
-        if (pool->def->source.dir == NULL) {
+        if (def->source.dir == NULL) {
             virReportError(VIR_ERR_INTERNAL_ERROR,
                            "%s", _("missing source path"));
             return -1;
         }
     } else {
-        if (pool->def->source.ndevice != 1) {
-            if (pool->def->source.ndevice == 0)
+        if (def->source.ndevice != 1) {
+            if (def->source.ndevice == 0)
                 virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                                _("missing source device"));
             else
@@ -264,22 +265,23 @@ virStorageBackendFileSystemIsValid(virStoragePoolObjPtr pool)
 static char *
 virStorageBackendFileSystemGetPoolSource(virStoragePoolObjPtr pool)
 {
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     char *src = NULL;
 
-    if (pool->def->type == VIR_STORAGE_POOL_NETFS) {
-        if (pool->def->source.format == VIR_STORAGE_POOL_NETFS_CIFS) {
+    if (def->type == VIR_STORAGE_POOL_NETFS) {
+        if (def->source.format == VIR_STORAGE_POOL_NETFS_CIFS) {
             if (virAsprintf(&src, "//%s/%s",
-                            pool->def->source.hosts[0].name,
-                            pool->def->source.dir) < 0)
+                            def->source.hosts[0].name,
+                            def->source.dir) < 0)
                 return NULL;
         } else {
             if (virAsprintf(&src, "%s:%s",
-                            pool->def->source.hosts[0].name,
-                            pool->def->source.dir) < 0)
+                            def->source.hosts[0].name,
+                            def->source.dir) < 0)
                 return NULL;
         }
     } else {
-        if (VIR_STRDUP(src, pool->def->source.devices[0].path) < 0)
+        if (VIR_STRDUP(src, def->source.devices[0].path) < 0)
             return NULL;
     }
     return src;
@@ -297,6 +299,7 @@ static int
 virStorageBackendFileSystemIsMounted(virStoragePoolObjPtr pool)
 {
     int ret = -1;
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     char *src = NULL;
     FILE *mtab;
     struct mntent ent;
@@ -317,8 +320,7 @@ virStorageBackendFileSystemIsMounted(virStoragePoolObjPtr pool)
         /* compare both mount destinations and sources to be sure the mounted
          * FS pool is really the one we're looking for
          */
-        if ((rc1 = virFileComparePaths(ent.mnt_dir,
-                                       pool->def->target.path)) < 0 ||
+        if ((rc1 = virFileComparePaths(ent.mnt_dir, def->target.path)) < 0 ||
             (rc2 = virFileComparePaths(ent.mnt_fsname, src)) < 0)
             goto cleanup;
 
@@ -349,16 +351,17 @@ virStorageBackendFileSystemIsMounted(virStoragePoolObjPtr pool)
 static int
 virStorageBackendFileSystemMount(virStoragePoolObjPtr pool)
 {
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     char *src = NULL;
     /* 'mount -t auto' doesn't seem to auto determine nfs (or cifs),
      *  while plain 'mount' does. We have to craft separate argvs to
      *  accommodate this */
-    bool netauto = (pool->def->type == VIR_STORAGE_POOL_NETFS &&
-                    pool->def->source.format == VIR_STORAGE_POOL_NETFS_AUTO);
-    bool glusterfs = (pool->def->type == VIR_STORAGE_POOL_NETFS &&
-                      pool->def->source.format == VIR_STORAGE_POOL_NETFS_GLUSTERFS);
-    bool cifsfs = (pool->def->type == VIR_STORAGE_POOL_NETFS &&
-                   pool->def->source.format == VIR_STORAGE_POOL_NETFS_CIFS);
+    bool netauto = (def->type == VIR_STORAGE_POOL_NETFS &&
+                    def->source.format == VIR_STORAGE_POOL_NETFS_AUTO);
+    bool glusterfs = (def->type == VIR_STORAGE_POOL_NETFS &&
+                      def->source.format == VIR_STORAGE_POOL_NETFS_GLUSTERFS);
+    bool cifsfs = (def->type == VIR_STORAGE_POOL_NETFS &&
+                   def->source.format == VIR_STORAGE_POOL_NETFS_CIFS);
     virCommandPtr cmd = NULL;
     int ret = -1;
     int rc;
@@ -371,7 +374,7 @@ virStorageBackendFileSystemMount(virStoragePoolObjPtr pool)
 
     /* Short-circuit if already mounted */
     if (rc == 1) {
-        VIR_INFO("Target '%s' is already mounted", pool->def->target.path);
+        VIR_INFO("Target '%s' is already mounted", def->target.path);
         return 0;
     }
 
@@ -381,34 +384,34 @@ virStorageBackendFileSystemMount(virStoragePoolObjPtr pool)
     if (netauto)
         cmd = virCommandNewArgList(MOUNT,
                                    src,
-                                   pool->def->target.path,
+                                   def->target.path,
                                    NULL);
     else if (glusterfs)
         cmd = virCommandNewArgList(MOUNT,
                                    "-t",
-                                   virStoragePoolFormatFileSystemNetTypeToString(pool->def->source.format),
+                                   virStoragePoolFormatFileSystemNetTypeToString(def->source.format),
                                    src,
                                    "-o",
                                    "direct-io-mode=1",
-                                   pool->def->target.path,
+                                   def->target.path,
                                    NULL);
     else if (cifsfs)
         cmd = virCommandNewArgList(MOUNT,
                                    "-t",
-                                   virStoragePoolFormatFileSystemNetTypeToString(pool->def->source.format),
+                                   virStoragePoolFormatFileSystemNetTypeToString(def->source.format),
                                    src,
-                                   pool->def->target.path,
+                                   def->target.path,
                                    "-o",
                                    "guest",
                                    NULL);
     else
         cmd = virCommandNewArgList(MOUNT,
                                    "-t",
-                                   (pool->def->type == VIR_STORAGE_POOL_FS ?
-                                    virStoragePoolFormatFileSystemTypeToString(pool->def->source.format) :
-                                    virStoragePoolFormatFileSystemNetTypeToString(pool->def->source.format)),
+                                   (def->type == VIR_STORAGE_POOL_FS ?
+                                    virStoragePoolFormatFileSystemTypeToString(def->source.format) :
+                                    virStoragePoolFormatFileSystemNetTypeToString(def->source.format)),
                                    src,
-                                   pool->def->target.path,
+                                   def->target.path,
                                    NULL);
 
     if (virCommandRun(cmd, NULL) < 0)
@@ -423,7 +426,6 @@ virStorageBackendFileSystemMount(virStoragePoolObjPtr pool)
 
 
 /**
- * @conn connection to report errors against
  * @pool storage pool to start
  *
  * Starts a directory or FS based storage pool.  The underlying source
@@ -432,10 +434,11 @@ virStorageBackendFileSystemMount(virStoragePoolObjPtr pool)
  * Returns 0 on success, -1 on error
  */
 static int
-virStorageBackendFileSystemStart(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                 virStoragePoolObjPtr pool)
+virStorageBackendFileSystemStart(virStoragePoolObjPtr pool)
 {
-    if (pool->def->type != VIR_STORAGE_POOL_DIR &&
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
+
+    if (def->type != VIR_STORAGE_POOL_DIR &&
         virStorageBackendFileSystemMount(pool) < 0)
         return -1;
 
@@ -444,7 +447,6 @@ virStorageBackendFileSystemStart(virConnectPtr conn ATTRIBUTE_UNUSED,
 
 
 /**
- * @conn connection to report errors against
  * @pool storage pool to unmount
  *
  * Stops a file storage pool.  The underlying source device is unmounted
@@ -456,9 +458,9 @@ virStorageBackendFileSystemStart(virConnectPtr conn ATTRIBUTE_UNUSED,
  * Returns 0 if successfully unmounted, -1 on error
  */
 static int
-virStorageBackendFileSystemStop(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                virStoragePoolObjPtr pool)
+virStorageBackendFileSystemStop(virStoragePoolObjPtr pool)
 {
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     virCommandPtr cmd = NULL;
     int ret = -1;
     int rc;
@@ -470,7 +472,7 @@ virStorageBackendFileSystemStop(virConnectPtr conn ATTRIBUTE_UNUSED,
     if ((rc = virStorageBackendFileSystemIsMounted(pool)) != 1)
         return rc;
 
-    cmd = virCommandNewArgList(UMOUNT, pool->def->target.path, NULL);
+    cmd = virCommandNewArgList(UMOUNT, def->target.path, NULL);
     if (virCommandRun(cmd, NULL) < 0)
         goto cleanup;
 
@@ -486,8 +488,10 @@ static int
 virStorageBackendFileSystemCheck(virStoragePoolObjPtr pool,
                                  bool *isActive)
 {
-    if (pool->def->type == VIR_STORAGE_POOL_DIR) {
-        *isActive = virFileExists(pool->def->target.path);
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
+
+    if (def->type == VIR_STORAGE_POOL_DIR) {
+        *isActive = virFileExists(def->target.path);
 #if WITH_STORAGE_FS
     } else {
         int ret;
@@ -561,25 +565,26 @@ static int
 virStorageBackendMakeFileSystem(virStoragePoolObjPtr pool,
                                 unsigned int flags)
 {
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     const char *device = NULL, *format = NULL;
     bool ok_to_mkfs = false;
     int ret = -1;
 
-    if (pool->def->source.devices == NULL) {
+    if (def->source.devices == NULL) {
         virReportError(VIR_ERR_OPERATION_INVALID,
                        _("No source device specified when formatting pool '%s'"),
-                       pool->def->name);
+                       def->name);
         goto error;
     }
 
-    device = pool->def->source.devices[0].path;
-    format = virStoragePoolFormatFileSystemTypeToString(pool->def->source.format);
+    device = def->source.devices[0].path;
+    format = virStoragePoolFormatFileSystemTypeToString(def->source.format);
     VIR_DEBUG("source device: '%s' format: '%s'", device, format);
 
     if (!virFileExists(device)) {
         virReportError(VIR_ERR_OPERATION_INVALID,
                        _("Source device does not exist when formatting pool '%s'"),
-                       pool->def->name);
+                       def->name);
         goto error;
     }
 
@@ -599,7 +604,6 @@ virStorageBackendMakeFileSystem(virStoragePoolObjPtr pool,
 
 
 /**
- * @conn connection to report errors against
  * @pool storage pool to build
  * @flags controls the pool formatting behaviour
  *
@@ -620,8 +624,7 @@ virStorageBackendMakeFileSystem(virStoragePoolObjPtr pool,
  * Returns 0 on success, -1 on error
  */
 static int
-virStorageBackendFileSystemBuild(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                 virStoragePoolObjPtr pool,
+virStorageBackendFileSystemBuild(virStoragePoolObjPtr pool,
                                  unsigned int flags)
 {
     virCheckFlags(VIR_STORAGE_POOL_BUILD_OVERWRITE |
@@ -779,9 +782,10 @@ virStorageFileBackendFileStat(virStorageSourcePtr src,
 
 
 static ssize_t
-virStorageFileBackendFileReadHeader(virStorageSourcePtr src,
-                                    ssize_t max_len,
-                                    char **buf)
+virStorageFileBackendFileRead(virStorageSourcePtr src,
+                              size_t offset,
+                              size_t len,
+                              char **buf)
 {
     int fd = -1;
     ssize_t ret = -1;
@@ -793,7 +797,14 @@ virStorageFileBackendFileReadHeader(virStorageSourcePtr src,
         return -1;
     }
 
-    if ((ret = virFileReadHeaderFD(fd, max_len, buf)) < 0) {
+    if (offset > 0) {
+        if (lseek(fd, offset, SEEK_SET) == (off_t) -1) {
+            virReportSystemError(errno, _("cannot seek into '%s'"), src->path);
+            goto cleanup;
+        }
+    }
+
+    if ((ret = virFileReadHeaderFD(fd, len, buf)) < 0) {
         virReportSystemError(errno,
                              _("cannot read header '%s'"), src->path);
         goto cleanup;
@@ -850,7 +861,7 @@ virStorageFileBackend virStorageFileBackendFile = {
     .storageFileCreate = virStorageFileBackendFileCreate,
     .storageFileUnlink = virStorageFileBackendFileUnlink,
     .storageFileStat = virStorageFileBackendFileStat,
-    .storageFileReadHeader = virStorageFileBackendFileReadHeader,
+    .storageFileRead = virStorageFileBackendFileRead,
     .storageFileAccess = virStorageFileBackendFileAccess,
     .storageFileChown = virStorageFileBackendFileChown,
 
@@ -865,7 +876,7 @@ virStorageFileBackend virStorageFileBackendBlock = {
     .backendDeinit = virStorageFileBackendFileDeinit,
 
     .storageFileStat = virStorageFileBackendFileStat,
-    .storageFileReadHeader = virStorageFileBackendFileReadHeader,
+    .storageFileRead = virStorageFileBackendFileRead,
     .storageFileAccess = virStorageFileBackendFileAccess,
     .storageFileChown = virStorageFileBackendFileChown,
 
@@ -900,13 +911,13 @@ virStorageBackendFsRegister(void)
         return -1;
 #endif /* WITH_STORAGE_FS */
 
-    if (virStorageBackendFileRegister(&virStorageFileBackendFile) < 0)
+    if (virStorageFileBackendRegister(&virStorageFileBackendFile) < 0)
         return -1;
 
-    if (virStorageBackendFileRegister(&virStorageFileBackendBlock) < 0)
+    if (virStorageFileBackendRegister(&virStorageFileBackendBlock) < 0)
         return -1;
 
-    if (virStorageBackendFileRegister(&virStorageFileBackendDir) < 0)
+    if (virStorageFileBackendRegister(&virStorageFileBackendDir) < 0)
         return -1;
 
     return 0;
