@@ -28,7 +28,7 @@
 #include "viralloc.h"
 #include "virerror.h"
 #include "virlog.h"
-#include "virstoragefile.h"
+#include "virstoragefilebackend.h"
 #include "virstring.h"
 #include "viruri.h"
 #include "storage_util.h"
@@ -74,8 +74,9 @@ static virStorageBackendGlusterStatePtr
 virStorageBackendGlusterOpen(virStoragePoolObjPtr pool)
 {
     virStorageBackendGlusterStatePtr ret = NULL;
-    const char *name = pool->def->source.name;
-    const char *dir = pool->def->source.dir;
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
+    const char *name = def->source.name;
+    const char *dir = def->source.dir;
     bool trailing_slash = true;
 
     /* Volume name must not contain '/'; optional path allows use of a
@@ -112,11 +113,11 @@ virStorageBackendGlusterOpen(virStoragePoolObjPtr pool)
         goto error;
     if (VIR_STRDUP(ret->uri->scheme, "gluster") < 0)
         goto error;
-    if (VIR_STRDUP(ret->uri->server, pool->def->source.hosts[0].name) < 0)
+    if (VIR_STRDUP(ret->uri->server, def->source.hosts[0].name) < 0)
         goto error;
     if (virAsprintf(&ret->uri->path, "/%s%s", ret->volname, ret->dir) < 0)
         goto error;
-    ret->uri->port = pool->def->source.hosts[0].port;
+    ret->uri->port = def->source.hosts[0].port;
 
     /* Actually connect to glfs */
     if (!(ret->vol = glfs_new(ret->volname))) {
@@ -151,20 +152,20 @@ virStorageBackendGlusterOpen(virStoragePoolObjPtr pool)
 
 
 static ssize_t
-virStorageBackendGlusterReadHeader(glfs_fd_t *fd,
-                                   const char *name,
-                                   ssize_t maxlen,
-                                   char **buf)
+virStorageBackendGlusterRead(glfs_fd_t *fd,
+                             const char *name,
+                             size_t len,
+                             char **buf)
 {
     char *s;
     size_t nread = 0;
 
-    if (VIR_ALLOC_N(*buf, maxlen) < 0)
+    if (VIR_ALLOC_N(*buf, len) < 0)
         return -1;
 
     s = *buf;
-    while (maxlen) {
-        ssize_t r = glfs_read(fd, s, maxlen, 0);
+    while (len) {
+        ssize_t r = glfs_read(fd, s, len, 0);
         if (r < 0 && errno == EINTR)
             continue;
         if (r < 0) {
@@ -175,7 +176,7 @@ virStorageBackendGlusterReadHeader(glfs_fd_t *fd,
         if (r == 0)
             return nread;
         s += r;
-        maxlen -= r;
+        len -= r;
         nread += r;
     }
     return nread;
@@ -246,7 +247,7 @@ virStorageBackendGlusterRefreshVol(virStorageBackendGlusterStatePtr state,
     glfs_fd_t *fd = NULL;
     virStorageSourcePtr meta = NULL;
     char *header = NULL;
-    ssize_t len = VIR_STORAGE_MAX_HEADER;
+    ssize_t len;
     int backingFormat;
 
     *volptr = NULL;
@@ -292,7 +293,8 @@ virStorageBackendGlusterRefreshVol(virStorageBackendGlusterStatePtr state,
         goto cleanup;
     }
 
-    if ((len = virStorageBackendGlusterReadHeader(fd, name, len, &header)) < 0)
+    if ((len = virStorageBackendGlusterRead(fd, name, VIR_STORAGE_MAX_HEADER,
+                                            &header)) < 0)
         goto cleanup;
 
     if (!(meta = virStorageFileGetMetadataFromBuf(name, header, len,
@@ -304,6 +306,8 @@ virStorageBackendGlusterRefreshVol(virStorageBackendGlusterStatePtr state,
         if (VIR_ALLOC(vol->target.backingStore) < 0)
             goto cleanup;
 
+        vol->target.backingStore->type = VIR_STORAGE_TYPE_NETWORK;
+
         vol->target.backingStore->path = meta->backingStoreRaw;
 
         if (backingFormat < 0)
@@ -338,10 +342,10 @@ virStorageBackendGlusterRefreshVol(virStorageBackendGlusterStatePtr state,
 }
 
 static int
-virStorageBackendGlusterRefreshPool(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                    virStoragePoolObjPtr pool)
+virStorageBackendGlusterRefreshPool(virStoragePoolObjPtr pool)
 {
     int ret = -1;
+    virStoragePoolDefPtr def = virStoragePoolObjGetDef(pool);
     virStorageBackendGlusterStatePtr state = NULL;
     struct {
         struct dirent ent;
@@ -385,8 +389,7 @@ virStorageBackendGlusterRefreshPool(virConnectPtr conn ATTRIBUTE_UNUSED,
 
         if (okay < 0)
             goto cleanup;
-        if (vol && VIR_APPEND_ELEMENT(pool->volumes.objs, pool->volumes.count,
-                                      vol) < 0)
+        if (vol && virStoragePoolObjAddVol(pool, vol) < 0)
             goto cleanup;
     }
     if (errno) {
@@ -401,11 +404,11 @@ virStorageBackendGlusterRefreshPool(virConnectPtr conn ATTRIBUTE_UNUSED,
         goto cleanup;
     }
 
-    pool->def->capacity = ((unsigned long long)sb.f_frsize *
-                           (unsigned long long)sb.f_blocks);
-    pool->def->available = ((unsigned long long)sb.f_bfree *
-                            (unsigned long long)sb.f_frsize);
-    pool->def->allocation = pool->def->capacity - pool->def->available;
+    def->capacity = ((unsigned long long)sb.f_frsize *
+                     (unsigned long long)sb.f_blocks);
+    def->available = ((unsigned long long)sb.f_bfree *
+                      (unsigned long long)sb.f_frsize);
+    def->allocation = def->capacity - def->available;
 
     ret = 0;
  cleanup:
@@ -419,8 +422,7 @@ virStorageBackendGlusterRefreshPool(virConnectPtr conn ATTRIBUTE_UNUSED,
 
 
 static int
-virStorageBackendGlusterVolDelete(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                  virStoragePoolObjPtr pool,
+virStorageBackendGlusterVolDelete(virStoragePoolObjPtr pool,
                                   virStorageVolDefPtr vol,
                                   unsigned int flags)
 {
@@ -481,8 +483,7 @@ virStorageBackendGlusterVolDelete(virConnectPtr conn ATTRIBUTE_UNUSED,
 
 
 static char *
-virStorageBackendGlusterFindPoolSources(virConnectPtr conn ATTRIBUTE_UNUSED,
-                                        const char *srcSpec,
+virStorageBackendGlusterFindPoolSources(const char *srcSpec,
                                         unsigned int flags)
 {
     virStoragePoolSourceList list = { .type = VIR_STORAGE_POOL_GLUSTER,
@@ -544,7 +545,7 @@ virStorageBackendGlusterCheckPool(virStoragePoolObjPtr pool,
     /* Return previous state remembered by the status XML. If the pool is not
      * available we will fail to refresh it and end up in the same situation.
      * This will save one attempt to open the connection to the remote server */
-    *active = pool->active;
+    *active = virStoragePoolObjIsActive(pool);
     return 0;
 }
 
@@ -574,9 +575,8 @@ virStorageFileBackendGlusterDeinit(virStorageSourcePtr src)
 {
     virStorageFileBackendGlusterPrivPtr priv = src->drv->priv;
 
-    VIR_DEBUG("deinitializing gluster storage file %p (gluster://%s:%s/%s%s)",
-              src, src->hosts->name, src->hosts->port ? src->hosts->port : "0",
-              src->volume, src->path);
+    VIR_DEBUG("deinitializing gluster storage file %p (gluster://%s:%u/%s%s)",
+              src, src->hosts->name, src->hosts->port, src->volume, src->path);
 
     if (priv->vol)
         glfs_fini(priv->vol);
@@ -598,15 +598,7 @@ virStorageFileBackendGlusterInitServer(virStorageFileBackendGlusterPrivPtr priv,
     case VIR_STORAGE_NET_HOST_TRANS_RDMA:
     case VIR_STORAGE_NET_HOST_TRANS_TCP:
         hoststr = host->name;
-
-        if (host->port &&
-            virStrToLong_i(host->port, NULL, 10, &port) < 0) {
-            virReportError(VIR_ERR_INTERNAL_ERROR,
-                           _("failed to parse port number '%s'"),
-                           host->port);
-            return -1;
-        }
-
+        port = host->port;
         break;
 
     case VIR_STORAGE_NET_HOST_TRANS_UNIX:
@@ -721,9 +713,10 @@ virStorageFileBackendGlusterStat(virStorageSourcePtr src,
 
 
 static ssize_t
-virStorageFileBackendGlusterReadHeader(virStorageSourcePtr src,
-                                       ssize_t max_len,
-                                       char **buf)
+virStorageFileBackendGlusterRead(virStorageSourcePtr src,
+                                 size_t offset,
+                                 size_t len,
+                                 char **buf)
 {
     virStorageFileBackendGlusterPrivPtr priv = src->drv->priv;
     glfs_fd_t *fd = NULL;
@@ -737,8 +730,16 @@ virStorageFileBackendGlusterReadHeader(virStorageSourcePtr src,
         return -1;
     }
 
-    ret = virStorageBackendGlusterReadHeader(fd, src->path, max_len, buf);
+    if (offset > 0) {
+        if (glfs_lseek(fd, offset, SEEK_SET) == (off_t) -1) {
+            virReportSystemError(errno, _("cannot seek into '%s'"), src->path);
+            goto cleanup;
+        }
+    }
 
+    ret = virStorageBackendGlusterRead(fd, src->path, len, buf);
+
+ cleanup:
     if (fd)
         glfs_close(fd);
 
@@ -818,7 +819,7 @@ virStorageFileBackendGlusterGetUniqueIdentifier(virStorageSourcePtr src)
                                                     priv)))
         return NULL;
 
-    ignore_value(virAsprintf(&priv->canonpath, "gluster://%s:%s/%s/%s",
+    ignore_value(virAsprintf(&priv->canonpath, "gluster://%s:%u/%s/%s",
                              src->hosts->name,
                              src->hosts->port,
                              src->volume,
@@ -851,7 +852,7 @@ virStorageFileBackend virStorageFileBackendGluster = {
     .storageFileCreate = virStorageFileBackendGlusterCreate,
     .storageFileUnlink = virStorageFileBackendGlusterUnlink,
     .storageFileStat = virStorageFileBackendGlusterStat,
-    .storageFileReadHeader = virStorageFileBackendGlusterReadHeader,
+    .storageFileRead = virStorageFileBackendGlusterRead,
     .storageFileAccess = virStorageFileBackendGlusterAccess,
     .storageFileChown = virStorageFileBackendGlusterChown,
 
@@ -865,7 +866,7 @@ virStorageBackendGlusterRegister(void)
     if (virStorageBackendRegister(&virStorageBackendGluster) < 0)
         return -1;
 
-    if (virStorageBackendFileRegister(&virStorageFileBackendGluster) < 0)
+    if (virStorageFileBackendRegister(&virStorageFileBackendGluster) < 0)
         return -1;
 
     return 0;
