@@ -46,7 +46,7 @@ testCompareXMLToXMLHelper(const void *data)
     ret = testCompareDomXML2XMLFiles(caps, xmlopt, xml_in,
                                      info->different ? xml_out : xml_in,
                                      !info->inactive_only,
-                                     NULL, NULL, info->parse_flags,
+                                     info->parse_flags,
                                      TEST_COMPARE_DOM_XML2XML_RESULT_SUCCESS);
  cleanup:
     VIR_FREE(xml_in);
@@ -66,13 +66,13 @@ mymain(void)
     if (!(xmlopt = lxcDomainXMLConfInit()))
         return EXIT_FAILURE;
 
-# define DO_TEST_FULL(name, is_different, inactive, parse_flags)        \
-    do {                                                                \
-        const struct testInfo info = {name, is_different, inactive,     \
-                                      parse_flags};                     \
-        if (virTestRun("LXC XML-2-XML " name,                           \
-                       testCompareXMLToXMLHelper, &info) < 0)           \
-            ret = -1;                                                   \
+# define DO_TEST_FULL(name, is_different, inactive, parse_flags) \
+    do { \
+        const struct testInfo info = {name, is_different, inactive, \
+                                      parse_flags}; \
+        if (virTestRun("LXC XML-2-XML " name, \
+                       testCompareXMLToXMLHelper, &info) < 0) \
+            ret = -1; \
     } while (0)
 
 # define DO_TEST(name) \
