@@ -62,6 +62,7 @@ qemuHotplugCreateObjects(virDomainXMLOptionPtr xmlopt,
 {
     int ret = -1;
     qemuDomainObjPrivatePtr priv = NULL;
+    const unsigned int parseFlags = 0;
 
     if (!(*vm = virDomainObjNew(xmlopt)))
         goto cleanup;
@@ -76,6 +77,7 @@ qemuHotplugCreateObjects(virDomainXMLOptionPtr xmlopt,
     virQEMUCapsSet(priv->qemuCaps, QEMU_CAPS_VIRTIO_CCW);
     virQEMUCapsSet(priv->qemuCaps, QEMU_CAPS_DEVICE_IVSHMEM_PLAIN);
     virQEMUCapsSet(priv->qemuCaps, QEMU_CAPS_DEVICE_IVSHMEM_DOORBELL);
+    virQEMUCapsSet(priv->qemuCaps, QEMU_CAPS_SCSI_DISK_WWN);
     if (event)
         virQEMUCapsSet(priv->qemuCaps, QEMU_CAPS_DEVICE_DEL_EVENT);
 
@@ -86,7 +88,7 @@ qemuHotplugCreateObjects(virDomainXMLOptionPtr xmlopt,
                                                driver.caps,
                                                driver.xmlopt,
                                                NULL,
-                                               VIR_DOMAIN_DEF_PARSE_INACTIVE)))
+                                               parseFlags)))
         goto cleanup;
 
     if (qemuDomainAssignAddresses((*vm)->def, priv->qemuCaps,
@@ -118,14 +120,17 @@ testQemuHotplugAttach(virDomainObjPtr vm,
         /* conn in only used for storage pool and secrets lookup so as long
          * as we don't use any of them, passing NULL should be safe
          */
-        ret = qemuDomainAttachDeviceDiskLive(NULL, &driver, vm, dev);
+        ret = qemuDomainAttachDeviceDiskLive(&driver, vm, dev);
         break;
     case VIR_DOMAIN_DEVICE_CHR:
-        ret = qemuDomainAttachChrDevice(NULL, &driver, vm, dev->data.chr);
+        ret = qemuDomainAttachChrDevice(&driver, vm, dev->data.chr);
         break;
     case VIR_DOMAIN_DEVICE_SHMEM:
         ret = qemuDomainAttachShmemDevice(&driver, vm, dev->data.shmem);
         break;
+    case VIR_DOMAIN_DEVICE_WATCHDOG:
+        ret = qemuDomainAttachWatchdog(&driver, vm, dev->data.watchdog);
+        break;
     default:
         VIR_TEST_VERBOSE("device type '%s' cannot be attached\n",
                 virDomainDeviceTypeToString(dev->type));
@@ -151,6 +156,9 @@ testQemuHotplugDetach(virDomainObjPtr vm,
     case VIR_DOMAIN_DEVICE_SHMEM:
         ret = qemuDomainDetachShmemDevice(&driver, vm, dev->data.shmem);
         break;
+    case VIR_DOMAIN_DEVICE_WATCHDOG:
+        ret = qemuDomainDetachWatchdog(&driver, vm, dev->data.watchdog);
+        break;
     default:
         VIR_TEST_VERBOSE("device type '%s' cannot be detached\n",
                 virDomainDeviceTypeToString(dev->type));
@@ -275,7 +283,8 @@ testQemuHotplug(const void *data)
 
     /* Now is the best time to feed the spoofed monitor with predefined
      * replies. */
-    if (!(test_mon = qemuMonitorTestNew(true, driver.xmlopt, vm, &driver, NULL)))
+    if (!(test_mon = qemuMonitorTestNew(true, driver.xmlopt, vm, &driver,
+                                        NULL, NULL)))
         goto cleanup;
 
     tmp = test->mon;
@@ -608,34 +617,34 @@ mymain(void)
     /* wait only 100ms for DEVICE_DELETED event */
     qemuDomainRemoveDeviceWaitTime = 100;
 
-#define DO_TEST(file, ACTION, dev, event, fial, kep, ...)                   \
-    do {                                                                    \
-        const char *my_mon[] = { __VA_ARGS__, NULL};                        \
-        const char *name = file " " #ACTION " " dev;                        \
-        data.action = ACTION;                                               \
-        data.domain_filename = file;                                        \
-        data.device_filename = dev;                                         \
-        data.fail = fial;                                                   \
-        data.mon = my_mon;                                                  \
-        data.keep = kep;                                                    \
-        data.deviceDeletedEvent = event;                                    \
-        if (virTestRun(name, testQemuHotplug, &data) < 0)                   \
-            ret = -1;                                                       \
+#define DO_TEST(file, ACTION, dev, event, fial, kep, ...) \
+    do { \
+        const char *my_mon[] = { __VA_ARGS__, NULL}; \
+        const char *name = file " " #ACTION " " dev; \
+        data.action = ACTION; \
+        data.domain_filename = file; \
+        data.device_filename = dev; \
+        data.fail = fial; \
+        data.mon = my_mon; \
+        data.keep = kep; \
+        data.deviceDeletedEvent = event; \
+        if (virTestRun(name, testQemuHotplug, &data) < 0) \
+            ret = -1; \
     } while (0)
 
-#define DO_TEST_ATTACH(file, dev, fial, kep, ...)                           \
+#define DO_TEST_ATTACH(file, dev, fial, kep, ...) \
     DO_TEST(file, ATTACH, dev, false, fial, kep, __VA_ARGS__)
 
-#define DO_TEST_DETACH(file, dev, fial, kep, ...)                           \
+#define DO_TEST_DETACH(file, dev, fial, kep, ...) \
     DO_TEST(file, DETACH, dev, false, fial, kep, __VA_ARGS__)
 
-#define DO_TEST_ATTACH_EVENT(file, dev, fial, kep, ...)                     \
+#define DO_TEST_ATTACH_EVENT(file, dev, fial, kep, ...) \
     DO_TEST(file, ATTACH, dev, true, fial, kep, __VA_ARGS__)
 
-#define DO_TEST_DETACH_EVENT(file, dev, fial, kep, ...)                     \
+#define DO_TEST_DETACH_EVENT(file, dev, fial, kep, ...) \
     DO_TEST(file, DETACH, dev, true, fial, kep, __VA_ARGS__)
 
-#define DO_TEST_UPDATE(file, dev, fial, kep, ...)                           \
+#define DO_TEST_UPDATE(file, dev, fial, kep, ...) \
     DO_TEST(file, UPDATE, dev, false, fial, kep, __VA_ARGS__)
 
 
@@ -643,16 +652,16 @@ mymain(void)
 #define HMP(msg)    "{\"return\": \"" msg "\"}"
 
 #define QMP_DEVICE_DELETED(dev) \
-    "{"                                                     \
-    "    \"timestamp\": {"                                  \
-    "        \"seconds\": 1374137171,"                      \
-    "        \"microseconds\": 2659"                        \
-    "    },"                                                \
-    "    \"event\": \"DEVICE_DELETED\","                    \
-    "    \"data\": {"                                       \
-    "        \"device\": \"" dev "\","                      \
-    "        \"path\": \"/machine/peripheral/" dev "\""     \
-    "    }"                                                 \
+    "{" \
+    "    \"timestamp\": {" \
+    "        \"seconds\": 1374137171," \
+    "        \"microseconds\": 2659" \
+    "    }," \
+    "    \"event\": \"DEVICE_DELETED\"," \
+    "    \"data\": {" \
+    "        \"device\": \"" dev "\"," \
+    "        \"path\": \"/machine/peripheral/" dev "\"" \
+    "    }" \
     "}\r\n"
 
     DO_TEST_UPDATE("graphics-spice", "graphics-spice-nochange", false, false, NULL);
@@ -809,16 +818,32 @@ mymain(void)
     DO_TEST_DETACH("base-live", "ivshmem-plain-detach", false, false,
                    "device_del", QMP_OK,
                    "object-del", QMP_OK);
+    DO_TEST_ATTACH("base-live+disk-scsi-wwn",
+                   "disk-scsi-duplicate-wwn", false, false,
+                   "human-monitor-command", HMP("OK\\r\\n"),
+                   "device_add", QMP_OK);
 
-#define DO_TEST_CPU_GROUP(prefix, vcpus, modernhp, expectfail)                 \
-    do {                                                                       \
-        cpudata.test = prefix;                                                 \
-        cpudata.newcpus = vcpus;                                               \
-        cpudata.modern = modernhp;                                             \
-        cpudata.fail = expectfail;                                             \
-        if (virTestRun("hotplug vcpus group " prefix,                          \
-                       testQemuHotplugCpuGroup, &cpudata) < 0)                 \
-            ret = -1;                                                          \
+    DO_TEST_ATTACH("base-live", "watchdog", false, true,
+                   "watchdog-set-action", QMP_OK,
+                   "device_add", QMP_OK);
+    DO_TEST_DETACH("base-live", "watchdog-full", false, false,
+                   "device_del", QMP_OK);
+
+    DO_TEST_ATTACH("base-live", "watchdog-user-alias", false, true,
+                   "watchdog-set-action", QMP_OK,
+                   "device_add", QMP_OK);
+    DO_TEST_DETACH("base-live", "watchdog-user-alias-full", false, false,
+                   "device_del", QMP_OK);
+
+#define DO_TEST_CPU_GROUP(prefix, vcpus, modernhp, expectfail) \
+    do { \
+        cpudata.test = prefix; \
+        cpudata.newcpus = vcpus; \
+        cpudata.modern = modernhp; \
+        cpudata.fail = expectfail; \
+        if (virTestRun("hotplug vcpus group " prefix, \
+                       testQemuHotplugCpuGroup, &cpudata) < 0) \
+            ret = -1; \
     } while (0)
 
     DO_TEST_CPU_GROUP("x86-modern-bulk", 7, true, false);
@@ -828,16 +853,16 @@ mymain(void)
     DO_TEST_CPU_GROUP("ppc64-modern-bulk", 23, true, true);
     DO_TEST_CPU_GROUP("ppc64-modern-bulk", 25, true, true);
 
-#define DO_TEST_CPU_INDIVIDUAL(prefix, mapstr, statefl, modernhp, expectfail)  \
-    do {                                                                       \
-        cpudata.test = prefix;                                                 \
-        cpudata.cpumap = mapstr;                                               \
-        cpudata.state = statefl;                                               \
-        cpudata.modern = modernhp;                                             \
-        cpudata.fail = expectfail;                                             \
-        if (virTestRun("hotplug vcpus group " prefix,                          \
-                       testQemuHotplugCpuIndividual, &cpudata) < 0)            \
-            ret = -1;                                                          \
+#define DO_TEST_CPU_INDIVIDUAL(prefix, mapstr, statefl, modernhp, expectfail) \
+    do { \
+        cpudata.test = prefix; \
+        cpudata.cpumap = mapstr; \
+        cpudata.state = statefl; \
+        cpudata.modern = modernhp; \
+        cpudata.fail = expectfail; \
+        if (virTestRun("hotplug vcpus group " prefix, \
+                       testQemuHotplugCpuIndividual, &cpudata) < 0) \
+            ret = -1; \
     } while (0)
 
     DO_TEST_CPU_INDIVIDUAL("x86-modern-individual-add", "7", true, true, false);
@@ -850,6 +875,7 @@ mymain(void)
     DO_TEST_CPU_INDIVIDUAL("ppc64-modern-individual", "17", true, true, true);
 
     qemuTestDriverFree(&driver);
+    virObjectUnref(data.vm);
     return (ret == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
