@@ -32,7 +32,6 @@
 #include "dirname.h"
 
 #include "storage/storage_driver.h"
-#include "storage/storage_source.h"
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
@@ -111,7 +110,6 @@ testStorageFileGetMetadata(const char *path,
     if (stat(path, &st) == 0) {
         if (S_ISDIR(st.st_mode)) {
             ret->type = VIR_STORAGE_TYPE_DIR;
-            ret->format = VIR_STORAGE_FILE_DIR;
         } else if (S_ISBLK(st.st_mode)) {
             ret->type = VIR_STORAGE_TYPE_BLOCK;
         }
@@ -310,8 +308,7 @@ static const char testStorageChainFormat[] =
     "type:%d\n"
     "format:%d\n"
     "protocol:%s\n"
-    "hostname:%s\n"
-    "secret:%s\n";
+    "hostname:%s\n";
 
 static int
 testStorageChain(const void *args)
@@ -357,7 +354,7 @@ testStorageChain(const void *args)
     }
 
     elt = meta;
-    while (elt) {
+    while (virStorageSourceIsBacking(elt)) {
         char *expect = NULL;
         char *actual = NULL;
 
@@ -376,8 +373,7 @@ testStorageChain(const void *args)
                         data->files[i]->type,
                         data->files[i]->format,
                         virStorageNetProtocolTypeToString(data->files[i]->protocol),
-                        NULLSTR(data->files[i]->hostname),
-                        NULLSTR(data->files[i]->secret)) < 0 ||
+                        NULLSTR(data->files[i]->hostname)) < 0 ||
             virAsprintf(&actual,
                         testStorageChainFormat, i,
                         NULLSTR(elt->path),
@@ -388,8 +384,7 @@ testStorageChain(const void *args)
                         elt->type,
                         elt->format,
                         virStorageNetProtocolTypeToString(elt->protocol),
-                        NULLSTR(elt->nhosts ? elt->hosts[0].name : NULL),
-                        NULLSTR(elt->auth ? elt->auth->username : NULL)) < 0) {
+                        NULLSTR(elt->nhosts ? elt->hosts[0].name : NULL)) < 0) {
             VIR_FREE(expect);
             VIR_FREE(actual);
             goto cleanup;
@@ -581,6 +576,7 @@ testPathRelativePrepare(void)
     size_t i;
 
     for (i = 0; i < ARRAY_CARDINALITY(backingchain); i++) {
+        backingchain[i].type = VIR_STORAGE_TYPE_FILE;
         if (i < ARRAY_CARDINALITY(backingchain) - 1)
             backingchain[i].backingStore = &backingchain[i + 1];
         else
@@ -693,7 +689,7 @@ testBackingParse(const void *args)
         goto cleanup;
     }
 
-    if (virDomainDiskSourceFormat(&buf, src, 0, 0) < 0 ||
+    if (virDomainDiskSourceFormat(&buf, src, 0, 0, NULL) < 0 ||
         !(xml = virBufferContentAndReset(&buf))) {
         fprintf(stderr, "failed to format disk source xml\n");
         goto cleanup;
@@ -740,28 +736,28 @@ mymain(void)
     if ((ret = testPrepImages()) != 0)
         return ret;
 
-#define TEST_ONE_CHAIN(start, format, flags, ...)                    \
-    do {                                                             \
-        size_t i;                                                    \
-        memset(&data, 0, sizeof(data));                              \
-        data = (struct testChainData){                               \
-            start, format, { __VA_ARGS__ }, 0, flags,                \
-        };                                                           \
-        for (i = 0; i < ARRAY_CARDINALITY(data.files); i++)          \
-            if (data.files[i])                                       \
-                data.nfiles++;                                       \
-        if (virTestRun(virTestCounterNext(),                         \
-                       testStorageChain, &data) < 0)                 \
-            ret = -1;                                                \
+#define TEST_ONE_CHAIN(start, format, flags, ...) \
+    do { \
+        size_t i; \
+        memset(&data, 0, sizeof(data)); \
+        data = (struct testChainData){ \
+            start, format, { __VA_ARGS__ }, 0, flags, \
+        }; \
+        for (i = 0; i < ARRAY_CARDINALITY(data.files); i++) \
+            if (data.files[i]) \
+                data.nfiles++; \
+        if (virTestRun(virTestCounterNext(), \
+                       testStorageChain, &data) < 0) \
+            ret = -1; \
     } while (0)
 
 #define VIR_FLATTEN_2(...) __VA_ARGS__
 #define VIR_FLATTEN_1(_1) VIR_FLATTEN_2 _1
 
-#define TEST_CHAIN(path, format, chain1, flags1, chain2, flags2)      \
-    do {                                                              \
-        TEST_ONE_CHAIN(path, format, flags1, VIR_FLATTEN_1(chain1));  \
-        TEST_ONE_CHAIN(path, format, flags2, VIR_FLATTEN_1(chain2));  \
+#define TEST_CHAIN(path, format, chain1, flags1, chain2, flags2) \
+    do { \
+        TEST_ONE_CHAIN(path, format, flags1, VIR_FLATTEN_1(chain1)); \
+        TEST_ONE_CHAIN(path, format, flags2, VIR_FLATTEN_1(chain2)); \
     } while (0)
 
     /* The actual tests, in several groups. */
@@ -963,7 +959,15 @@ mymain(void)
         .type = VIR_STORAGE_TYPE_DIR,
         .format = VIR_STORAGE_FILE_DIR,
     };
-    TEST_CHAIN(absdir, VIR_STORAGE_FILE_AUTO,
+    testFileData dir_as_raw = {
+        .path = canondir,
+        .type = VIR_STORAGE_TYPE_DIR,
+        .format = VIR_STORAGE_FILE_RAW,
+    };
+    TEST_CHAIN(absdir, VIR_STORAGE_FILE_RAW,
+               (&dir_as_raw), EXP_PASS,
+               (&dir_as_raw), ALLOW_PROBE | EXP_PASS);
+    TEST_CHAIN(absdir, VIR_STORAGE_FILE_NONE,
                (&dir), EXP_PASS,
                (&dir), ALLOW_PROBE | EXP_PASS);
     TEST_CHAIN(absdir, VIR_STORAGE_FILE_DIR,
@@ -1102,17 +1106,17 @@ mymain(void)
     chain2 = chain->backingStore;
     chain3 = chain2->backingStore;
 
-#define TEST_LOOKUP_TARGET(id, target, from, name, index, result,       \
-                           meta, parent)                                \
-    do {                                                                \
-        data2 = (struct testLookupData){                                \
-            chain, target, from, name, index,                           \
-            result, meta, parent, };                                    \
-        if (virTestRun("Chain lookup " #id,                             \
-                       testStorageLookup, &data2) < 0)                  \
-            ret = -1;                                                   \
+#define TEST_LOOKUP_TARGET(id, target, from, name, index, result, \
+                           meta, parent) \
+    do { \
+        data2 = (struct testLookupData){ \
+            chain, target, from, name, index, \
+            result, meta, parent, }; \
+        if (virTestRun("Chain lookup " #id, \
+                       testStorageLookup, &data2) < 0) \
+            ret = -1; \
     } while (0)
-#define TEST_LOOKUP(id, from, name, result, meta, parent)               \
+#define TEST_LOOKUP(id, from, name, result, meta, parent) \
     TEST_LOOKUP_TARGET(id, NULL, from, name, 0, result, meta, parent)
 
     TEST_LOOKUP(0, NULL, "bogus", NULL, NULL, NULL);
@@ -1243,13 +1247,13 @@ mymain(void)
     TEST_LOOKUP_TARGET(80, "vda", chain3, "vda[2]", 2, NULL, NULL, NULL);
     TEST_LOOKUP_TARGET(81, "vda", NULL, "vda[3]", 3, NULL, NULL, NULL);
 
-#define TEST_PATH_CANONICALIZE(id, PATH, EXPECT)                            \
-    do {                                                                    \
-        data3.path = PATH;                                                  \
-        data3.expect = EXPECT;                                              \
-        if (virTestRun("Path canonicalize " #id,                            \
-                       testPathCanonicalize, &data3) < 0)                   \
-            ret = -1;                                                       \
+#define TEST_PATH_CANONICALIZE(id, PATH, EXPECT) \
+    do { \
+        data3.path = PATH; \
+        data3.expect = EXPECT; \
+        if (virTestRun("Path canonicalize " #id, \
+                       testPathCanonicalize, &data3) < 0) \
+            ret = -1; \
     } while (0)
 
     TEST_PATH_CANONICALIZE(1, "/", "/");
@@ -1286,14 +1290,14 @@ mymain(void)
     TEST_PATH_CANONICALIZE(30, "/cycle2/link", NULL);
     TEST_PATH_CANONICALIZE(31, "///", "/");
 
-#define TEST_RELATIVE_BACKING(id, TOP, BASE, EXPECT)                        \
-    do {                                                                    \
-        data4.top = &TOP;                                                   \
-        data4.base = &BASE;                                                 \
-        data4.expect = EXPECT;                                              \
-        if (virTestRun("Path relative resolve " #id,                        \
-                       testPathRelative, &data4) < 0)                       \
-            ret = -1;                                                       \
+#define TEST_RELATIVE_BACKING(id, TOP, BASE, EXPECT) \
+    do { \
+        data4.top = &TOP; \
+        data4.base = &BASE; \
+        data4.expect = EXPECT; \
+        if (virTestRun("Path relative resolve " #id, \
+                       testPathRelative, &data4) < 0) \
+            ret = -1; \
     } while (0)
 
     testPathRelativePrepare();
@@ -1336,13 +1340,13 @@ mymain(void)
 
     virTestCounterReset("Backing store parse ");
 
-#define TEST_BACKING_PARSE(bck, xml)                                           \
-    do {                                                                       \
-        data5.backing = bck;                                                   \
-        data5.expect = xml;                                                    \
-        if (virTestRun(virTestCounterNext(),                                   \
-                       testBackingParse, &data5) < 0)                          \
-            ret = -1;                                                          \
+#define TEST_BACKING_PARSE(bck, xml) \
+    do { \
+        data5.backing = bck; \
+        data5.expect = xml; \
+        if (virTestRun(virTestCounterNext(), \
+                       testBackingParse, &data5) < 0) \
+            ret = -1; \
     } while (0)
 
     TEST_BACKING_PARSE("path", "<source file='path'/>\n");
@@ -1484,6 +1488,15 @@ mymain(void)
                        "<source protocol='nbd' name='blah'>\n"
                        "  <host name='example.org' port='6000'/>\n"
                        "</source>\n");
+    TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"nbd\","
+                                       "\"server\": { \"type\":\"unix\","
+                                                     "\"path\":\"/path/socket\""
+                                                   "}"
+                                      "}"
+                            "}",
+                       "<source protocol='nbd'>\n"
+                       "  <host transport='unix' socket='/path/socket'/>\n"
+                       "</source>\n");
     TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"ssh\","
                                        "\"host\":\"example.org\","
                                        "\"port\":\"6000\","
@@ -1558,12 +1571,32 @@ mymain(void)
                                        "\"transport\":\"tcp\","
                                        "\"portal\":\"test.org:1234\","
                                        "\"target\":\"iqn.2016-12.com.virttest:emulated-iscsi-noauth.target\","
-                                       "\"lun\":6"
+                                       "\"lun\":\"6\""
                                       "}"
                             "}",
                        "<source protocol='iscsi' name='iqn.2016-12.com.virttest:emulated-iscsi-noauth.target/6'>\n"
                        "  <host name='test.org' port='1234'/>\n"
                        "</source>\n");
+    TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"iscsi\","
+                                       "\"transport\":\"tcp\","
+                                       "\"portal\":\"[2001::0]:1234\","
+                                       "\"target\":\"iqn.2016-12.com.virttest:emulated-iscsi-noauth.target\","
+                                       "\"lun\":6"
+                                      "}"
+                            "}",
+                       "<source protocol='iscsi' name='iqn.2016-12.com.virttest:emulated-iscsi-noauth.target/6'>\n"
+                       "  <host name='[2001::0]' port='1234'/>\n"
+                       "</source>\n");
+    TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"iscsi\","
+                                       "\"transport\":\"tcp\","
+                                       "\"portal\":\"[2001::0]\","
+                                       "\"target\":\"iqn.2016-12.com.virttest:emulated-iscsi-noauth.target\","
+                                       "\"lun\":6"
+                                      "}"
+                            "}",
+                       "<source protocol='iscsi' name='iqn.2016-12.com.virttest:emulated-iscsi-noauth.target/6'>\n"
+                       "  <host name='[2001::0]' port='3260'/>\n"
+                       "</source>\n");
     TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"sheepdog\","
                                        "\"vdi\":\"test\","
                                        "\"server\":{ \"type\":\"inet\","
@@ -1585,6 +1618,16 @@ mymain(void)
                        "<source protocol='sheepdog' name='Alice'>\n"
                        "  <host name='10.10.10.10' port='7000'/>\n"
                        "</source>\n");
+    TEST_BACKING_PARSE("json:{\"file\":{\"driver\":\"vxhs\","
+                                       "\"vdisk-id\":\"c6718f6b-0401-441d-a8c3-1f0064d75ee0\","
+                                       "\"server\": {  \"host\":\"example.com\","
+                                                      "\"port\":\"9999\""
+                                                   "}"
+                                      "}"
+                            "}",
+                       "<source protocol='vxhs' name='c6718f6b-0401-441d-a8c3-1f0064d75ee0'>\n"
+                       "  <host name='example.com' port='9999'/>\n"
+                       "</source>\n");
 #endif /* WITH_YAJL */
 
  cleanup:
