@@ -34,12 +34,14 @@
 #include "virtime.h"
 #include "conf/network_conf.h"
 
-#define VIRSH_COMMON_OPT_NETWORK                              \
-    {.name = "network",                                       \
-     .type = VSH_OT_DATA,                                     \
-     .flags = VSH_OFLAG_REQ,                                  \
-     .help = N_("network name or uuid")                       \
-    }                                                         \
+#define VIRSH_COMMON_OPT_NETWORK(cflags) \
+    {.name = "network", \
+     .type = VSH_OT_DATA, \
+     .flags = VSH_OFLAG_REQ, \
+     .help = N_("network name or uuid"), \
+     .completer = virshNetworkNameCompleter, \
+     .completer_flags = cflags, \
+    }
 
 virNetworkPtr
 virshCommandOptNetworkBy(vshControl *ctl, const vshCmd *cmd,
@@ -93,7 +95,7 @@ static const vshCmdInfo info_network_autostart[] = {
 };
 
 static const vshCmdOptDef opts_network_autostart[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(VIR_CONNECT_LIST_NETWORKS_PERSISTENT),
     {.name = "disable",
      .type = VSH_OT_BOOL,
      .help = N_("disable autostarting")
@@ -240,7 +242,7 @@ static const vshCmdInfo info_network_destroy[] = {
 };
 
 static const vshCmdOptDef opts_network_destroy[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(VIR_CONNECT_LIST_NETWORKS_ACTIVE),
     {.name = NULL}
 };
 
@@ -279,7 +281,7 @@ static const vshCmdInfo info_network_dumpxml[] = {
 };
 
 static const vshCmdOptDef opts_network_dumpxml[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(0),
     {.name = "inactive",
      .type = VSH_OT_BOOL,
      .help = N_("show inactive defined XML")
@@ -330,7 +332,7 @@ static const vshCmdInfo info_network_info[] = {
 };
 
 static const vshCmdOptDef opts_network_info[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(0),
     {.name = NULL}
 };
 
@@ -647,8 +649,8 @@ static const vshCmdOptDef opts_network_list[] = {
     {.name = NULL}
 };
 
-#define FILTER(NAME, FLAG)              \
-    if (vshCommandOptBool(cmd, NAME))   \
+#define FILTER(NAME, FLAG) \
+    if (vshCommandOptBool(cmd, NAME)) \
         flags |= (FLAG)
 static bool
 cmdNetworkList(vshControl *ctl, const vshCmd *cmd ATTRIBUTE_UNUSED)
@@ -779,7 +781,7 @@ static const vshCmdInfo info_network_start[] = {
 };
 
 static const vshCmdOptDef opts_network_start[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(VIR_CONNECT_LIST_NETWORKS_INACTIVE),
     {.name = NULL}
 };
 
@@ -817,7 +819,7 @@ static const vshCmdInfo info_network_undefine[] = {
 };
 
 static const vshCmdOptDef opts_network_undefine[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(VIR_CONNECT_LIST_NETWORKS_PERSISTENT),
     {.name = NULL}
 };
 
@@ -856,7 +858,7 @@ static const vshCmdInfo info_network_update[] = {
 };
 
 static const vshCmdOptDef opts_network_update[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(0),
     {.name = "command",
      .type = VSH_OT_DATA,
      .flags = VSH_OFLAG_REQ,
@@ -1057,7 +1059,7 @@ static const vshCmdInfo info_network_edit[] = {
 };
 
 static const vshCmdOptDef opts_network_edit[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(0),
     {.name = NULL}
 };
 
@@ -1090,12 +1092,12 @@ cmdNetworkEdit(vshControl *ctl, const vshCmd *cmd)
         goto cleanup;
 
 #define EDIT_GET_XML virshNetworkGetXMLDesc(network)
-#define EDIT_NOT_CHANGED                                                     \
-    do {                                                                     \
+#define EDIT_NOT_CHANGED \
+    do { \
         vshPrintExtra(ctl, _("Network %s XML configuration not changed.\n"), \
-                      virNetworkGetName(network));                           \
-        ret = true;                                                          \
-        goto edit_cleanup;                                                   \
+                      virNetworkGetName(network)); \
+        ret = true; \
+        goto edit_cleanup; \
     } while (0)
 #define EDIT_DEFINE \
     (network_edited = virNetworkDefineXML(priv->conn, doc_edited))
@@ -1304,7 +1306,7 @@ static const vshCmdInfo info_network_dhcp_leases[] = {
 };
 
 static const vshCmdOptDef opts_network_dhcp_leases[] = {
-    VIRSH_COMMON_OPT_NETWORK,
+    VIRSH_COMMON_OPT_NETWORK(0),
     {.name = "mac",
      .type = VSH_OT_STRING,
      .flags = VSH_OFLAG_NONE,
