@@ -34,97 +34,113 @@
 #include "virstring.h"
 #include "virtime.h"
 
-#define VIRSH_COMMON_OPT_POOL_FULL                            \
-    VIRSH_COMMON_OPT_POOL(N_("pool name or uuid"))            \
+#define VIRSH_COMMON_OPT_POOL_FULL(cflags) \
+    VIRSH_COMMON_OPT_POOL(N_("pool name or uuid"), cflags)
 
-#define VIRSH_COMMON_OPT_POOL_BUILD                           \
-    {.name = "build",                                         \
-     .type = VSH_OT_BOOL,                                     \
-     .flags = 0,                                              \
-     .help = N_("build the pool as normal")                   \
-    }                                                         \
+#define VIRSH_COMMON_OPT_POOL_BUILD \
+    {.name = "build", \
+     .type = VSH_OT_BOOL, \
+     .flags = 0, \
+     .help = N_("build the pool as normal") \
+    }
 
-#define VIRSH_COMMON_OPT_POOL_NO_OVERWRITE                        \
-    {.name = "no-overwrite",                                      \
-     .type = VSH_OT_BOOL,                                         \
-     .flags = 0,                                                  \
-     .help = N_("do not overwrite any existing data")             \
-    }                                                             \
+#define VIRSH_COMMON_OPT_POOL_NO_OVERWRITE \
+    {.name = "no-overwrite", \
+     .type = VSH_OT_BOOL, \
+     .flags = 0, \
+     .help = N_("do not overwrite any existing data") \
+    }
 
-#define VIRSH_COMMON_OPT_POOL_OVERWRITE                           \
-    {.name = "overwrite",                                         \
-     .type = VSH_OT_BOOL,                                         \
-     .flags = 0,                                                  \
-     .help = N_("overwrite any existing data")                    \
-    }                                                             \
+#define VIRSH_COMMON_OPT_POOL_OVERWRITE \
+    {.name = "overwrite", \
+     .type = VSH_OT_BOOL, \
+     .flags = 0, \
+     .help = N_("overwrite any existing data") \
+    }
 
-#define VIRSH_COMMON_OPT_POOL_X_AS                                     \
-    {.name = "name",                                                   \
-     .type = VSH_OT_DATA,                                              \
-     .flags = VSH_OFLAG_REQ,                                           \
-     .help = N_("name of the pool")                                    \
-    },                                                                 \
-    {.name = "type",                                                   \
-     .type = VSH_OT_DATA,                                              \
-     .flags = VSH_OFLAG_REQ,                                           \
-     .help = N_("type of the pool")                                    \
-    },                                                                 \
-    {.name = "print-xml",                                              \
-     .type = VSH_OT_BOOL,                                              \
-     .help = N_("print XML document, but don't define/create")         \
-    },                                                                 \
-    {.name = "source-host",                                            \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("source-host for underlying storage")                  \
-    },                                                                 \
-    {.name = "source-path",                                            \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("source path for underlying storage")                  \
-    },                                                                 \
-    {.name = "source-dev",                                             \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("source device for underlying storage")                \
-    },                                                                 \
-    {.name = "source-name",                                            \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("source name for underlying storage")                  \
-    },                                                                 \
-    {.name = "target",                                                 \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("target for underlying storage")                       \
-    },                                                                 \
-    {.name = "source-format",                                          \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("format for underlying storage")                       \
-    },                                                                 \
-    {.name = "auth-type",                                              \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("auth type to be used for underlying storage")         \
-    },                                                                 \
-    {.name = "auth-username",                                          \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("auth username to be used for underlying storage")     \
-    },                                                                 \
-    {.name = "secret-usage",                                           \
-     .type = VSH_OT_STRING,                                            \
+#define VIRSH_COMMON_OPT_POOL_X_AS \
+    {.name = "name", \
+     .type = VSH_OT_DATA, \
+     .flags = VSH_OFLAG_REQ, \
+     .help = N_("name of the pool") \
+    }, \
+    {.name = "type", \
+     .type = VSH_OT_DATA, \
+     .flags = VSH_OFLAG_REQ, \
+     .help = N_("type of the pool") \
+    }, \
+    {.name = "print-xml", \
+     .type = VSH_OT_BOOL, \
+     .help = N_("print XML document, but don't define/create") \
+    }, \
+    {.name = "source-host", \
+     .type = VSH_OT_STRING, \
+     .help = N_("source-host for underlying storage") \
+    }, \
+    {.name = "source-path", \
+     .type = VSH_OT_STRING, \
+     .help = N_("source path for underlying storage") \
+    }, \
+    {.name = "source-dev", \
+     .type = VSH_OT_STRING, \
+     .help = N_("source device for underlying storage") \
+    }, \
+    {.name = "source-name", \
+     .type = VSH_OT_STRING, \
+     .help = N_("source name for underlying storage") \
+    }, \
+    {.name = "target", \
+     .type = VSH_OT_STRING, \
+     .help = N_("target for underlying storage") \
+    }, \
+    {.name = "source-format", \
+     .type = VSH_OT_STRING, \
+     .help = N_("format for underlying storage") \
+    }, \
+    {.name = "auth-type", \
+     .type = VSH_OT_STRING, \
+     .help = N_("auth type to be used for underlying storage") \
+    }, \
+    {.name = "auth-username", \
+     .type = VSH_OT_STRING, \
+     .help = N_("auth username to be used for underlying storage") \
+    }, \
+    {.name = "secret-usage", \
+     .type = VSH_OT_STRING, \
      .help = N_("auth secret usage to be used for underlying storage") \
-    },                                                                 \
-    {.name = "adapter-name",                                           \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("adapter name to be used for underlying storage")      \
-    },                                                                 \
-    {.name = "adapter-wwnn",                                           \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("adapter wwnn to be used for underlying storage")      \
-    },                                                                 \
-    {.name = "adapter-wwpn",                                           \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("adapter wwpn to be used for underlying storage")      \
-    },                                                                 \
-    {.name = "adapter-parent",                                         \
-     .type = VSH_OT_STRING,                                            \
-     .help = N_("adapter parent to be used for underlying storage")    \
-    }                                                                  \
+    }, \
+    {.name = "secret-uuid", \
+     .type = VSH_OT_STRING, \
+     .help = N_("auth secret UUID to be used for underlying storage") \
+    }, \
+    {.name = "adapter-name", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter name to be used for underlying storage") \
+    }, \
+    {.name = "adapter-wwnn", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter wwnn to be used for underlying storage") \
+    }, \
+    {.name = "adapter-wwpn", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter wwpn to be used for underlying storage") \
+    }, \
+    {.name = "adapter-parent", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter parent scsi_hostN to be used for underlying vHBA storage") \
+    }, \
+    {.name = "adapter-parent-wwnn", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter parent scsi_hostN wwnn to be used for underlying vHBA storage") \
+    }, \
+    {.name = "adapter-parent-wwpn", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter parent scsi_hostN wwpn to be used for underlying vHBA storage") \
+    }, \
+    {.name = "adapter-parent-fabric-wwn", \
+     .type = VSH_OT_STRING, \
+     .help = N_("adapter parent scsi_hostN fabric_wwn to be used for underlying vHBA storage") \
+    }
 
 virStoragePoolPtr
 virshCommandOptPoolBy(vshControl *ctl, const vshCmd *cmd, const char *optname,
@@ -139,6 +155,9 @@ virshCommandOptPoolBy(vshControl *ctl, const vshCmd *cmd, const char *optname,
     if (vshCommandOptStringReq(ctl, cmd, optname, &n) < 0)
         return NULL;
 
+    if (cmd->skipChecks && !n)
+        return NULL;
+
     vshDebug(ctl, VSH_ERR_INFO, "%s: found option <%s>: %s\n",
              cmd->def->name, optname, n);
 
@@ -178,7 +197,7 @@ static const vshCmdInfo info_pool_autostart[] = {
 };
 
 static const vshCmdOptDef opts_pool_autostart[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(VIR_CONNECT_LIST_STORAGE_POOLS_PERSISTENT),
 
     {.name = "disable",
      .type = VSH_OT_BOOL,
@@ -302,9 +321,13 @@ virshBuildPoolXML(vshControl *ctl,
                *srcDev = NULL, *srcName = NULL, *srcFormat = NULL,
                *target = NULL, *authType = NULL, *authUsername = NULL,
                *secretUsage = NULL, *adapterName = NULL, *adapterParent = NULL,
-               *adapterWwnn = NULL, *adapterWwpn = NULL;
+               *adapterWwnn = NULL, *adapterWwpn = NULL, *secretUUID = NULL,
+               *adapterParentWwnn = NULL, *adapterParentWwpn = NULL,
+               *adapterParentFabricWwn = NULL;
     virBuffer buf = VIR_BUFFER_INITIALIZER;
 
+    VSH_EXCLUSIVE_OPTIONS("secret-usage", "secret-uuid");
+
     if (vshCommandOptStringReq(ctl, cmd, "name", &name) < 0)
         goto cleanup;
     if (vshCommandOptStringReq(ctl, cmd, "type", &type) < 0)
@@ -319,10 +342,14 @@ virshBuildPoolXML(vshControl *ctl,
         vshCommandOptStringReq(ctl, cmd, "auth-type", &authType) < 0 ||
         vshCommandOptStringReq(ctl, cmd, "auth-username", &authUsername) < 0 ||
         vshCommandOptStringReq(ctl, cmd, "secret-usage", &secretUsage) < 0 ||
+        vshCommandOptStringReq(ctl, cmd, "secret-uuid", &secretUUID) < 0 ||
         vshCommandOptStringReq(ctl, cmd, "adapter-name", &adapterName) < 0 ||
         vshCommandOptStringReq(ctl, cmd, "adapter-wwnn", &adapterWwnn) < 0 ||
         vshCommandOptStringReq(ctl, cmd, "adapter-wwpn", &adapterWwpn) < 0 ||
-        vshCommandOptStringReq(ctl, cmd, "adapter-parent", &adapterParent) < 0)
+        vshCommandOptStringReq(ctl, cmd, "adapter-parent", &adapterParent) < 0 ||
+        vshCommandOptStringReq(ctl, cmd, "adapter-parent-wwnn", &adapterParentWwnn) < 0 ||
+        vshCommandOptStringReq(ctl, cmd, "adapter-parent-wwpn", &adapterParentWwpn) < 0 ||
+        vshCommandOptStringReq(ctl, cmd, "adapter-parent-fabric-wwn", &adapterParentFabricWwn) < 0)
         goto cleanup;
 
     virBufferAsprintf(&buf, "<pool type='%s'>\n", type);
@@ -343,17 +370,26 @@ virshBuildPoolXML(vshControl *ctl,
             virBufferAddLit(&buf, "<adapter type='fc_host'");
             if (adapterParent)
                 virBufferAsprintf(&buf, " parent='%s'", adapterParent);
+            else if (adapterParentWwnn && adapterParentWwpn)
+                virBufferAsprintf(&buf, " parent_wwnn='%s' parent_wwnn='%s'",
+                                  adapterParentWwnn, adapterParentWwpn);
+            else if (adapterParentFabricWwn)
+                virBufferAsprintf(&buf, " parent_fabric_wwn='%s'",
+                                  adapterParentFabricWwn);
             virBufferAsprintf(&buf, " wwnn='%s' wwpn='%s'/>\n",
                               adapterWwnn, adapterWwpn);
         } else if (adapterName) {
             virBufferAsprintf(&buf, "<adapter type='scsi_host' name='%s'/>\n",
                               adapterName);
         }
-        if (authType && authUsername && secretUsage) {
+        if (authType && authUsername && (secretUsage || secretUUID)) {
             virBufferAsprintf(&buf, "<auth type='%s' username='%s'>\n",
                               authType, authUsername);
             virBufferAdjustIndent(&buf, 2);
-            virBufferAsprintf(&buf, "<secret usage='%s'/>\n", secretUsage);
+            if (secretUsage)
+                virBufferAsprintf(&buf, "<secret usage='%s'/>\n", secretUsage);
+            else
+                virBufferAsprintf(&buf, "<secret uuid='%s'/>\n", secretUUID);
             virBufferAdjustIndent(&buf, -2);
             virBufferAddLit(&buf, "</auth>\n");
         }
@@ -565,7 +601,7 @@ static const vshCmdInfo info_pool_build[] = {
 };
 
 static const vshCmdOptDef opts_pool_build[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
     VIRSH_COMMON_OPT_POOL_NO_OVERWRITE,
     VIRSH_COMMON_OPT_POOL_OVERWRITE,
 
@@ -615,7 +651,7 @@ static const vshCmdInfo info_pool_destroy[] = {
 };
 
 static const vshCmdOptDef opts_pool_destroy[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(VIR_CONNECT_LIST_STORAGE_POOLS_ACTIVE),
 
     {.name = NULL}
 };
@@ -655,7 +691,7 @@ static const vshCmdInfo info_pool_delete[] = {
 };
 
 static const vshCmdOptDef opts_pool_delete[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(VIR_CONNECT_LIST_STORAGE_POOLS_INACTIVE),
 
     {.name = NULL}
 };
@@ -695,7 +731,7 @@ static const vshCmdInfo info_pool_refresh[] = {
 };
 
 static const vshCmdOptDef opts_pool_refresh[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = NULL}
 };
@@ -735,7 +771,7 @@ static const vshCmdInfo info_pool_dumpxml[] = {
 };
 
 static const vshCmdOptDef opts_pool_dumpxml[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = "inactive",
      .type = VSH_OT_BOOL,
@@ -1626,7 +1662,7 @@ static const vshCmdInfo info_pool_info[] = {
 };
 
 static const vshCmdOptDef opts_pool_info[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = "bytes",
      .type = VSH_OT_BOOL,
@@ -1716,7 +1752,7 @@ static const vshCmdInfo info_pool_name[] = {
 };
 
 static const vshCmdOptDef opts_pool_name[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = NULL}
 };
@@ -1748,7 +1784,7 @@ static const vshCmdInfo info_pool_start[] = {
 };
 
 static const vshCmdOptDef opts_pool_start[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(VIR_CONNECT_LIST_STORAGE_POOLS_INACTIVE),
     VIRSH_COMMON_OPT_POOL_BUILD,
     VIRSH_COMMON_OPT_POOL_NO_OVERWRITE,
     VIRSH_COMMON_OPT_POOL_OVERWRITE,
@@ -1809,7 +1845,7 @@ static const vshCmdInfo info_pool_undefine[] = {
 };
 
 static const vshCmdOptDef opts_pool_undefine[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(VIR_CONNECT_LIST_STORAGE_POOLS_PERSISTENT),
 
     {.name = NULL}
 };
@@ -1849,7 +1885,7 @@ static const vshCmdInfo info_pool_uuid[] = {
 };
 
 static const vshCmdOptDef opts_pool_uuid[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = NULL}
 };
@@ -1886,7 +1922,7 @@ static const vshCmdInfo info_pool_edit[] = {
 };
 
 static const vshCmdOptDef opts_pool_edit[] = {
-    VIRSH_COMMON_OPT_POOL_FULL,
+    VIRSH_COMMON_OPT_POOL_FULL(0),
 
     {.name = NULL}
 };
@@ -1918,12 +1954,12 @@ cmdPoolEdit(vshControl *ctl, const vshCmd *cmd)
     }
 
 #define EDIT_GET_XML virStoragePoolGetXMLDesc(pool, flags)
-#define EDIT_NOT_CHANGED                                                     \
-    do {                                                                     \
-        vshPrintExtra(ctl, _("Pool %s XML configuration not changed.\n"),    \
-                 virStoragePoolGetName(pool));                               \
-        ret = true;                                                          \
-        goto edit_cleanup;                                                   \
+#define EDIT_NOT_CHANGED \
+    do { \
+        vshPrintExtra(ctl, _("Pool %s XML configuration not changed.\n"), \
+                 virStoragePoolGetName(pool)); \
+        ret = true; \
+        goto edit_cleanup; \
     } while (0)
 #define EDIT_DEFINE \
     (pool_edited = virStoragePoolDefineXML(priv->conn, doc_edited, 0))
@@ -1952,7 +1988,9 @@ VIR_ENUM_IMPL(virshPoolEvent,
               N_("Defined"),
               N_("Undefined"),
               N_("Started"),
-              N_("Stopped"))
+              N_("Stopped"),
+              N_("Created"),
+              N_("Deleted"))
 
 static const char *
 virshPoolEventToString(int event)
